<?php

class ModulesSwitchListener
{
    private PriceByUserRoleCoreObject $_object;
    private $_pluginsPath;
    private WooUserRolePricesFestiPlugin $_ecommercePlugin;

    const CORE_PRICES_PLUGIN = 'PriceByUserRoleCore';
    const TAXES_PLUGIN = 'UserRoleTaxes';
    const IMPORT_PRODUCTS_PLUGIN = 'WooImportProducts';
    const PRODUCTS_EXPORT_PLUGIN = 'UserRoleProductsExporter';
    const QUANTITY_DISCOUNT_PLUGIN = 'QuantityDiscount';

    public function __construct(
        Core $core,
        PriceByUserRoleCoreObject $object,
        WooUserRolePricesFestiPlugin $ecommercePlugin
    )
    {
        $this->_object = $object;

        $instance = $this;
        $core->addEventListener(
            EngineFacade::FILTER_BACKEND_MENU_OPTIONS,
            //array($this, 'onBackendMenuItems')
            function (FestiEvent &$event) use ($instance) {
                try {
                    $instance->onBackendMenuItems($event);
                } catch (Exception $exp) {
                    EngineFacade::getInstance()->handleSystemError($exp);
                }
            }
        );

        $this->_pluginsPath = $core->getOption('plugins_path');
        $this->_ecommercePlugin = $ecommercePlugin;
    } // end __construct

    public function onBackendMenuItems(FestiEvent &$event)
    {
        $menuItems = &$event->getTarget();

        if ($this->_isModuleActive(static::TAXES_PLUGIN)) {
            $menuItems['taxesTab'] = 'Tax Options';
        }

        if ($this->_isModuleActive(static::QUANTITY_DISCOUNT_PLUGIN)) {
            $menuItems['quantityDiscountTab'] = 'Quantity Discount';
        }

        if ($this->_isModuleActive(static::IMPORT_PRODUCTS_PLUGIN)) {
            $menuItems['importProductTab'] = 'Import Products';
        }

        $menuItems['registrationTab'] = 'Product License';
        $menuItems['extensionsTab'] = 'Extensions';
    } // end onBackendMenuItems

    private function _isModuleActive(string $name): bool
    {
        return $this->_ecommercePlugin->isModuleExist($name) &&
               $this->_object->isModuleActive($name);
    } // end _isModuleActive
}
