<?php

/**
 * Plugin Name: Quantity discount for Prices by User Role
*/

class QuantityDiscountPlugin extends DisplayPlugin
{
    private $_ecommercePlugin;
    private $_ecommerceFacade;

    public static $cartItemsCount;

    /**
     * @override
     */
    public function onInit()
    {
        parent::onInit();

        $this->_ecommercePlugin =$GLOBALS[PRICE_BY_ROLE_PLUGIN_GLOBAL_NAME];

        $this->_ecommerceFacade = EcommerceFactory::getInstance();

        EngineFacade::getInstance()->addActionListener(
            'woocommerce_single_product_summary',
            array($this, 'onDisplayQuantityDiscountText')
        );
    } // end __construct

    public function getQuantityDiscountByUserRole($price)
    {
        $plugin = $this->_ecommercePlugin;

        $settings = $plugin->getSettings();
        $userRole = $plugin->getUserRole();

        $settings = $plugin->getPreparedQuantityDiscountOptions($settings);

        $key = SettingsWooUserRolePrices::QUANTITY_DISCOUNT_OPTION_KEY;

        if (!isset($settings[$key])) {
            return $price;
        }

        $quantityDiscount = $settings[$key];

        if (!$plugin->hasUserRoleInOptions(
            $userRole,
            $quantityDiscount
        )) {
            return $price;
        }

        $minValue = $quantityDiscount[$userRole]['minValue'];
        $maxValue = $quantityDiscount[$userRole]['maxValue'];
        $amount = $quantityDiscount[$userRole]['value'];

        if ($this->_isCartItemsCountBetweenQuantityDiscountRange(
            $minValue,
            $maxValue,
            $amount
        )) {
            $price = $this->_getPriceWithQuantityDiscount(
                $price,
                $amount,
                $plugin
            );
        }

        return $price;
    } // end getQuantityDiscountByUserRole

    public function onDisplayQuantityDiscountText(): void
    {
        if ($this->isDisplayQuantityDiscountText()) {

            $vars = array(
                'text' => $this->getQuantityDiscountText()
            );

            echo $this->_ecommercePlugin->fetch('frontend/quantity_discount_text.phtml', $vars);
        }
    } // end onDisplayQuantityDiscountText

    public function getSettingsQuantityDiscountByUserRole($userRole): array
    {
        $result = array();

        if (!$userRole) {
            return $result;
        }

        $settings = $this->_ecommercePlugin->getSettings();

        $quantity_discount_option_key = SettingsWooUserRolePrices::QUANTITY_DISCOUNT_OPTION_KEY;

        if (isset($settings[$quantity_discount_option_key][$userRole])) {
            $result = $settings[$quantity_discount_option_key][$userRole];
        }

        return $result;
    } // end getSettingsQuantityDiscountByUserRole

    public function isDisplayQuantityDiscountText(): bool
    {
        $plugin = $this->_ecommercePlugin;
        $userRole = $plugin->getUserRole();

        if (!$userRole) {
            return false;
        }

        $settings = $plugin->getSettings();

        $userRoleSettings = $this->getSettingsQuantityDiscountByUserRole($userRole);

        return isset($settings['displayQuantityDiscountText']) && !empty($settings['displayQuantityDiscountText']) &&
            $userRoleSettings &&
            isset($userRoleSettings['isVisible']) && !empty($userRoleSettings['isVisible']) &&
            isset($userRoleSettings['minValue']) &&
            isset($userRoleSettings['maxValue']) &&
            isset($userRoleSettings['value']) &&
            ($userRoleSettings['minValue'] > 0 || $userRoleSettings['maxValue'] > 0) &&
            ($userRoleSettings['value'] > 0);
    } // end isDisplayQuantityDiscountText

    public function getQuantityDiscountText(): string
    {
        $userRole = $this->_ecommercePlugin->getUserRole();

        if (!$userRole) {
            return '';
        }

        $settings = $this->getSettingsQuantityDiscountByUserRole($userRole);

        if (!$settings) {
            return '';
        }

        $minValue = isset($settings['minValue']) ? $settings['minValue'] : 0;
        $maxValue = isset($settings['maxValue']) ? $settings['maxValue'] : 0;

        if ($minValue > 0 && $maxValue > 0) {
            return StringManagerWooUserRolePrices::getWord(
                sprintf(
                    'Wholesale price from %d to %d',
                    $minValue, $maxValue
                )
            );
        }

        if ($minValue > 0) {
            return StringManagerWooUserRolePrices::getWord(
                sprintf(
                    'Wholesale price from %d',
                    $minValue
                )
            );
        }

        if ($maxValue > 0) {
            return StringManagerWooUserRolePrices::getWord(
                sprintf(
                    'Wholesale price to %d',
                    $maxValue
                )
            );
        }

        return '';
    } // end getQuantityDiscountText

    private function _getPriceWithQuantityDiscount($price, $amount, $plugin)
    {
        $key = SettingsWooUserRolePrices::QUANTITY_DISCOUNT_OPTION_KEY;

        if ($plugin->isPercentDiscountType($key)) {
            $amount = $plugin->getAmountOfDiscountOrMarkUpInPercentage(
                $price,
                $amount
            );
        }

        $minimalPrice = PRICE_BY_ROLE_PRODUCT_MINIMAL_PRICE;
        $price = ($amount > $price) ? $minimalPrice : $price - $amount;

        $numberOfDecimals = $this->_ecommerceFacade->getNumberOfDecimals();

        if (!$numberOfDecimals) {
            $price = round($price);
        }

        return $price;
    } // end _getPriceWithQuantityDiscount

    private function _isCartItemsCountBetweenQuantityDiscountRange(
        $minValue,
        $maxValue,
        $amount
    )
    {
        return $amount &&
               self::$cartItemsCount &&
               self::$cartItemsCount >= $minValue &&
               self::$cartItemsCount <= $maxValue;
    } // end _isCartItemsCountBetweenQuantityDiscountRange
}