<?php

declare(strict_types=1);

use ast\Node;
use Phan\CodeBase;
use Phan\Language\Context;
use Phan\Language\Element\Clazz;
use Phan\Language\UnionType;
use Phan\Language\Element\Method;
use Phan\Language\Element\Property;
use Phan\Language\Element\Func;
use Phan\Language\Element\FunctionInterface;

use Phan\PluginV3;
use Phan\PluginV3\AnalyzeFunctionCallCapability;

use Phan\Language\Type;


class CorePlugin extends PluginV3 implements AnalyzeFunctionCallCapability
{
    public function getAnalyzeFunctionCallClosures(CodeBase $code_base): array
    {
        $closureCallback = static function (
            CodeBase $codeBase,
            Context $context,
            Method $method,
            array $args,
            ?Node $node
        ): void {

            $pluginClassName = "\\".$args[0].'Plugin';

            $realReturnType = UnionType::fromStringInContext($pluginClassName, $context, Type::FROM_TYPE);

            $method->setRealReturnType($realReturnType);

            $analyzer = static function (
                CodeBase $codeBase,
                Context $context,
                FunctionInterface $function,
                array $args
            ) use ($realReturnType) : UnionType {
                return $realReturnType;
            };

            $method->setDependentReturnTypeClosure($analyzer);

        };

        return [
            '\\Core::getPluginInstance' => $closureCallback,
        ];
    }
}

return new CorePlugin();
