<?php
// phpcs:ignoreFile


class CoreAutoload
{
    public function __construct()
    {
        $instance = $this;
        spl_autoload_register(function ($className) use ($instance) {
            $instance->onAutoload($className);
        });

        require_once __DIR__.'/locale/Utils.php';
    }

    protected function onAutoload(string $className): void
    {
        // XXX: Temporary dirty hack
        if (strpos($className, "core\\") !== false) {
            $this->onCoreClassLoad($className);
        } else if (strpos($className, "plugin\\") !== false) {
            $this->onPluginClassLoad($className);
        } else if (strpos($className, "\\") !== false) {
            $this->onDefaultLoad($className);
        }
    }

    protected function onCoreClassLoad(string $className): void
    {
        $relativeClassPath = str_replace("core\\", "", $className);
        $relativeClassPath = str_replace("\\", DIRECTORY_SEPARATOR, $relativeClassPath);
        //$relativeClassPath = str_replace(__NAMESPACE__.DIRECTORY_SEPARATOR, "", $relativeClassPath);

        $path = __DIR__.DIRECTORY_SEPARATOR.$relativeClassPath.'.php';

        if (file_exists($path)) {
            require_once $path;
        }
    } // end onCoreClassLoad

    protected function onPluginClassLoad(string $className): void
    {
        $basePath = Core::getInstance()->getOption(Core::OPTION_PLUGINS_PATH);

        $chunks = explode('\\', $className);
        $chunks = array_slice($chunks, 1);

        $path = $basePath.join(DIRECTORY_SEPARATOR, $chunks).'.php';
        if (file_exists($path)) {
            require_once $path;
        }
    } // end onPluginClassLoad

    protected function onDefaultLoad(string $className): void
    {
        $relativeClassPath = str_replace("\\", DIRECTORY_SEPARATOR, $className);

        $basePath = Core::getInstance()->getOption(Core::OPTION_PATH);

        $path = $basePath.$relativeClassPath.'.php';
        if (file_exists($path)) {
            require_once $path;
        }
    } // end onDefaultLoad

}

return new CoreAutoload();