<?php

use core\store\event\UrlStoreActionEvent;
use core\store\IStore;

class StoreStageBlock extends StageBlock
{
    /**
     * @var IStore
     */
    protected $instance;

    /**
     * @override
     */
    protected function onInit(): void
    {
        if (!($this->instance instanceof IStore)) {
            throw new StageException(
                'Undefined store stage block instance type.'
            );
        }

        assert($this->instance instanceof EventDispatcher);
        $this->instance->addEventListener(
            Store::EVENT_PREPARE_ACTION_URL,
            array($this, 'onPrepareActionUrl')
        );
    } // end onInit

    /**
     * @event Store::EVENT_PREPARE_ACTION_URL
     * @param UrlStoreActionEvent $event
     */
    public function onPrepareActionUrl(UrlStoreActionEvent &$event)
    {
        $params = &$event->getParams();

        $externalParams = $_REQUEST;
        
        $params[StageBuilder::REQUEST_KEY_BLOCK_INDEX] = $this->getStageIndex();
        
        foreach ($externalParams as $key => $value) {
            if (array_key_exists($key, $params)) {
                continue;
            }
            $params[$key] = $value;
        }
    } // end onPrepareActionUrl
    
   /**
    * @override
    */
    protected function onRequest(Response &$response)
    {
        $this->instance->onRequest($response);
        
        if (Response::isAjaxRequest()) {
            return true;
        }
        
        return $response->getContent();
    }

    public function isExec(): bool
    {
        $actionName = $this->instance->getActionNameFromRequest();

        $action = $this->instance->createActionInstance($actionName);

        return $action->isExec();
    } // end isExec
}