<?php

class CheckboxField extends AbstractField
{
    private const BOOL_VALUE_TRUE  = '1';
    private const BOOL_VALUE_FALSE = '0';

    /**
     * @override
     */
    public function getEditInput(?string $value = '', $inline = null): ?string
    {
        $readonly = $this->get('readonly');
        if ($readonly) {
            return $this->displayValue($value);
        }
        
        if (is_null($value) && $this->get('value')) {
            $value = $this->get('value');
        }
        
        if (is_numeric($value) || is_bool($value)) {
            $checked = ($value) ? 'checked' : '';
        } else {
            $checked = (strtoupper(substr($value, 0, 1)) == 'Y') ? 'checked' : '';
        }

        $this->assign('checked', $checked);
        $this->assign('value', htmlspecialchars($value));

        return $this->fetch('edit.phtml');
    }
    
    /**
     * @override
     */
    public function displayValue(?string $value, array $row = null): ?string
    {
        $this->assign('value', $value);
        
        return $this->fetch('cell_value.phtml');
    }

    /**
     * @override
     */
    public function getValue(array $request = array())
    {
        $value = parent::getValue($request);

        if ($this->_isBool() && !is_null($value)) {
            $value = $this->_getBoolValue($value);
        }
        
        return $value;
    } // end getValue

    private function _isBool(): bool
    {
        $isBool = $this->get('isbool');

        return $isBool === true || $isBool === 'true';
    }

    /**
     * @override
     */
    public function displayRO($value): ?string
    {
        return $this->displayValue($value);
    }
    
    protected function getFilterTemplateName(): string
    {
        return 'checkbox.phtml';
    } // end getFilterTemplateName
    
    /**
     * @override
     */
    protected function getTemplatePath(): string
    {
        return parent::getTemplatePath().'checkbox'.DIRECTORY_SEPARATOR;
    } // end getTemplatePath    
    
    private function _getBoolValue($value): string
    {
        if ($value) {
            return static::BOOL_VALUE_TRUE;
        }
        
        return static::BOOL_VALUE_FALSE;
    } // end _getBoolValue
}
