<?php

class PercentField extends AbstractField
{
    /**
     * @override
     */
    public function getEditInput(?string $value = '', $inline = null): ?string
    {
        if (empty($value)) {
            $value = $this->get('isnull') ? '' : 0;
        }

        $this->assign('value', $value);

        return $this->fetch('edit.phtml');
    } // end getEditInput

    /**
     * @override
     */
    public function onInit(FieldModel $scheme): void
    {
        parent::onInit($scheme);
        
        $regExp = $this->get('regexp');
        if (!$regExp) {
            $this->set('regexp', '^[\+\-0-9]*\.{0,1}[0-9]{0,2}$');
        }
        
        $mask = $this->get('mask');
        if (!$mask) {
            $this->set('mask', "'mask': '9{0,6}.{0,1}9{0,2}'");
        }
    } // end onInit

    /**
     * @override
     */
    public function getInfoValue($value, $row = null): ?string
    {
        $value = $this->displayValue($value, $row);
        return sprintf('%s%%', $value);
    }

    /**
     * override
     */
    protected function getDefaultCellValue(?string $value, array $row = array()): string
    {
        if (empty($value) && !$this->get('isnull')) {
            $value = 0;
        }

        $this->assign('currentValue', $value);

        return $this->fetch('cell_value.phtml');
    } // end getDefaultCellValue
    
    /**
     * @override
     */
    protected function getTemplatePath(): string
    {
        return parent::getTemplatePath().'percent'.DIRECTORY_SEPARATOR;
    } // end getTemplatePath
}