<?php

class SelectField extends AbstractSelectField
{
    public const OPTION_ID    = 'id';
    public const OPTION_VALUE = 'value';
    public array $valuesList = array();

    /**
     * @override
     */
    public function onInit(FieldModel $scheme): void
    {
        parent::onInit($scheme);
        
        $this->onInitOptions($scheme);
    } // end onInit
    
    /**
     * @return array
     */
    public function getValuesList(): array
    {
        return $this->valuesList;
    } // end getValuesList
    
    /**
     * @override
     */
    protected function onInitOptions($scheme): void
    {
        if (!$scheme->hasOptions()) {
            return;
        }

        $options = $scheme->getOptions();
        
        foreach ($options as $item) {
            $id = $item[static::OPTION_ID];
            
            $hasPermission = true;
            
            if (array_key_exists(static::OPTION_PERMISSION, $item) && $item[static::OPTION_PERMISSION]) {
                $systemPlugin = Core::getInstance()->getSystemPlugin();

                $hasPermission = $systemPlugin->hasUserPermissionToSection($item[static::OPTION_PERMISSION]);
            }
            
            if ($hasPermission) {
                $this->valuesList[$id] = $item[static::OPTION_VALUE];
            }
        }
    } // end onInitOptions
    
    /**
     * @override 
     * @param $value
     * @return string|null
     */
    public function getFormattedValue($value): ?string
    {
        $result = $value;
        
        if (array_key_exists($value, $this->valuesList)) {
            $result = $this->valuesList[$value];
        }
    
        return parent::getFormattedValue($result);
    } // end getFormattedValue

    /**
     * @override
     */
    public function getEditInput(?string $value = '', $inline = null): ?string
    {
        if (!empty($this->attributes['readonly'])) {
            return $this->displayRO($value);
        }

        $this->assign('value', $value);

        if ($this->get('ajaxChild') && $inline) {
            $ajaxChildValues = $this->getAjaxChildValues($inline);
            $this->assign('ajaxChildValues', $ajaxChildValues);
        }

        return $this->fetch('edit.phtml');
    }

    /**
     * @override
     */
    public function displayRO($value): string
    {
        // FIXME:
        if (empty($value)) {
            $attrValue = $this->get('value');
            
            if ($attrValue !== false) {
                $value = $attrValue;
            }
        }
        
        if (empty($value)) {
            return '';
        }

        $value = $this->displayValue($value);
        
        return $value;
    }

    /**
     * @override
     */
    public function displayValue(?string $value, array $row = null): ?string
    {
        if (is_null($value)) {
            $value = 'NULL';
        }

        if (array_key_exists($value, $this->valuesList)) {
            $value = $this->valuesList[$value];
        } else {
            $value = "";
        }
        
        return $value;
    } // end displayValue

    /**
     * @override
     */
    protected function onFilterFetch(): void
    {
        $this->filterValues = $this->valuesList;
    } // end onFilterFetch
    
    /**
     * @override
     */
    protected function getTemplatePath(): string
    {
        return parent::getTemplatePath().'select'.DIRECTORY_SEPARATOR;
    } // end getTemplatePath
    
}