<?php

class SerializeField extends AbstractField
{
    protected array $defaultValues = array();

    /**
     * @override
     */
    public function onInit(FieldModel $scheme): void
    {
        parent::onInit($scheme);
        
        if (!$scheme->hasOptions()) {
            return;
        }
        
        $options = $scheme->getOptions();
        foreach ($options as $item) {
            $this->defaultValues[$item['id']] = $item['value'];
        }
    } // end onInit
    
    /**
     * @override
     */
    public function getEditInput(?string $value = '', $inline = null): ?string
    {
        $values = $this->_decodeValue($value);
        $values = array_merge($this->defaultValues, $values);
        $this->assign('values', $values);

        return $this->fetch('edit.phtml');
    }
    
    /**
     * @override
     */
    public function displayValue(?string $value, array $row = array()): ?string
    {
        $values = $this->_decodeValue($value);
        $this->assign('values', $values);
        
        return $this->fetch('cell_value.phtml');
    }

    /**
     * @override
     */
    public function getInfoValue($value, $row = null): ?string
    {
        return $this->displayValue($value, $row);
    }

    /**
     * @override
     */
    public function getValue(array $request = array())
    {
        $value = parent::getValue($request);

        if (!$value && $this->get('isnull')) {
            $value = null;
            return $value;
        }
    
        if ($this->_isRequired($value)) {
            $this->lastErrorMessage = __l(
                '%s is required field',
                $this->attributes['caption']
            );
            return false;
        }

        if (!$value) {
            return '';
        }
        
        $value = array_combine($value['key'], $value['value']);
        $value = array_filter($value);

        return json_encode($value);
    } // end getValue
    
    private function _decodeValue($value)
    {
        return !is_null($value) ? json_decode($value, true) : array();
    } // end _decodeValue
    
    /**
     * @override
     */
    protected function getTemplatePath(): string
    {
        return parent::getTemplatePath().'serialize'.DIRECTORY_SEPARATOR;
    } // end getTemplatePath

    protected function onPrepareValue(&$value)
    {
        $keys      = array_keys($value);
        $keyValues = array_shift($value);
        $values    = array_shift($value);
        
        $this->_onPrepareValues($keyValues);
        $this->_onPrepareValues($values);

        $value = array(
            array_shift($keys) => $keyValues,
            array_shift($keys) => $values,
        );
        
        return true;
    } // end onPrepareValue
    
    private function _onPrepareValues(&$values)
    {
        if (!is_array($values)) {
            return true;
        }
        
        foreach ($values as &$value) {
            $value = filter_var($value, FILTER_SANITIZE_STRING);
        }

        return true;
    } // end _onPreparedValue
    
    private function _isRequired($value)
    {
        $isRequired = $this->get('required');
        
        return $isRequired && 
               is_array($value) && 
               array_key_exists('key', $value) &&
               !current($value['key']);
    }
}