<?php

class TextField extends AbstractField
{
    public const OPTION_MAX_CHARS = 'maxChars';

    /**
     * Returns true if field has maxlength.
     *
     * @return bool
     */
    public function hasMaxCharsLimit(): bool
    {
        return (bool) $this->get(static::OPTION_MAX_CHARS);
    } // end hasMaxCharsLimit

    /**
     * @override
     */
    public function getEditInput(?string $value = '', $inline = null): ?string
    {
        $this->assign('value', $value);

        return $this->fetch(
            'text'.DIRECTORY_SEPARATOR.'edit.phtml'
        );
    } // end getEditInput

    /**
     * @override
     */
    protected function getAttributesFields(): array
    {
        $fields = parent::getAttributesFields();

        $fields[static::OPTION_MAX_CHARS] = static::FIELD_TYPE_INT;

        return $fields;
    } // end getAttributesFields

    /**
     * @override
     */
    public function isValidValue($value): bool
    {
        $res = parent::isValidValue($value);
        if (!$res) {
            return false;
        }

        $maxChars = $this->get(static::OPTION_MAX_CHARS);
        if ($maxChars > 0 && mb_strlen($value) > $maxChars) {
            $this->lastErrorMessage = __l(
                '%s cannot be longer than %s characters.',
                $this->attributes['caption'],
                $maxChars
            );

            return false;
        }

        return true;
    } // end isValidValue

    /**
     * @override
     */
    public function getElementAttributes(): string
    {
        $attributes = parent::getElementAttributes();

        $maxChars = $this->get(static::OPTION_MAX_CHARS);
        if ($maxChars !== false) {
            $attributes .= sprintf(' maxlength="%s"', $maxChars);
        }

        return $attributes;
    } // end getElementAttributes
}
