<?php

use core\store\field\IStoreField;
use core\store\field\IStoreForeignField;

class CassandraProxy extends SqlProxy
{

    public function loadListValues(bool $isAllColumns = false): array
    {
        $columns = $this->getQueryColumns($isAllColumns);
        $where = $this->getQueryWhere();

        $fromTable = $this->getOriginalStoreName();

        $sql = 'SELECT ' . join(', ', $columns['current']) . ' FROM ' . $fromTable;

        if ($where) {
            $sql .= ' WHERE ' . join(' AND ', $where);
        }

        $orderBy = $this->getQueryOrderDirection();
        if ($where && $orderBy) {
            $sql .= ' ORDER BY ' . $orderBy;
        }

        if ($this->isUseLimit()) {
            $rowsPerPage = $this->store->getRowsPerPageCount();
            $sql .= ' LIMIT ' . $rowsPerPage;
        }

        $rows = $this->connection->getAll($sql);

        $totalQuery = 'SELECT COUNT(*) FROM ' . $fromTable;
        if ($where) {
            $totalQuery .= ' WHERE ' . join(' AND ', $where);
        }
        $total = (int) $this->connection->getOne($totalQuery);
        $this->setCount($total);

        return $rows;
    }

    public function getQueryColumns(bool $isAllColumns = false): array
    {
        $fields = array(
            'current' => array(),
            'foreign' => array(),
            'join'    => array()
        );

        $primaryKey = $this->store->getPrimaryKey();

        $fields['current'][] = $primaryKey;

        $this->_getQueryColumnsFromFields($fields, $isAllColumns);

        /*
        $parentColumnName = $this->store->getParentFieldName();
        if ($parentColumnName) {
            echo __METHOD__;
            var_dump($parentColumnName);
            die();
            $fields[] = $storeName . static::STORE_NAME_AND_COLUMN_SEPARATOR . $parentColumnName;
        }
        */

        return $fields;
    } // end getQueryColumns

    private function _getQueryColumnsFromFields(array &$fields, bool $isAllColumns = false): array
    {
        foreach ($this->model->getFields() as $field) {

            assert($field instanceof IStoreField);

            if ($this->isHiddenColumnInList($field, $isAllColumns)) {
                continue;
            }

            if ($field instanceof IStoreForeignField) {
                $storeName = $field->getForeignStoreName();
                $key = $field->getForeignFieldKey();
                $value = $field->getForeignFieldValue();

                $fields['foreign'][$storeName][$key][$value] = $field->getName();
                $fields['current'][] = $field->getName();
            } else if ($this->store->getName() == $field->getStoreName()) {
                $fields['current'][] = $field->getName();
            } else {
                $storeName = $field->getStoreName();
                $routers = $this->model->getRouters();
                var_dump($routers);
                die();
                //$fields['join'][$storeName][$key][$value] = $value;
            }
        }

        return $fields;
    } // end _getQueryColumnsFromFields

    public function loadForeignAssigns($primaryValue, array $options): array
    {
        die(__METHOD__);
        die(__METHOD__);
        // TODO: Implement loadForeignAssigns() method.
    }

    public function loadForeignValues(array $options): array
    {
        die(__METHOD__);
        // TODO: Implement loadForeignValues() method.
    }

    public function loadForeignKeyValues(IStoreForeignField &$fields): bool
    {
        die(__METHOD__);
        // TODO: Implement loadForeignKeyValues() method.
    }

    public function updateManyToManyValues(Many2manyField $item, int $id, array $values): bool
    {
        die(__METHOD__);
        // TODO: Implement updateManyToManyValues() method.
    }

    public function removeAllManyToManyValuesByPrimaryKey($primaryKeyValue): bool
    {
        die(__METHOD__);
        // TODO: Implement removeAllManyToManyValuesByPrimaryKey() method.
    }


    public function getQueryJoins(array $columns): array
    {
        die(__METHOD__);
        // TODO: Implement getQueryJoins() method.
    }

    public function loadAggregations(): array
    {
        die(__METHOD__);
        // TODO: Implement loadAggregations() method.
    }

    public function createAuditTable(string $auditTableName, string $originalTableName): bool
    {
        die(__METHOD__);
        // TODO: Implement createAuditTable() method.
    }

    public function isBegin(): bool
    {
        die(__METHOD__);
        // TODO: Implement isBegin() method.
    }

    public function begin(): bool
    {
        die(__METHOD__);
        // TODO: Implement begin() method.
    }

    public function commit(): void
    {
        die(__METHOD__);
        // TODO: Implement commit() method.
    }

    public function rollback(): void
    {
        die(__METHOD__);
        // TODO: Implement rollback() method.
    }

    public function insert(array $values)
    {
        die(__METHOD__);
        // TODO: Implement insert() method.
    }

    public function updateByPrimaryKey($primaryKeyValue, array $values): bool
    {
        die(__METHOD__);
        // TODO: Implement updateByPrimaryKey() method.
    }

    public function search(array $search): array
    {
        die(__METHOD__);
        // TODO: Implement search() method.
    }

    public function loadRow(array $search, bool $isAllColumns = false): ?array
    {
        die(__METHOD__);
        // TODO: Implement loadRow() method.
    }

    public function removeByPrimaryKey(string $primaryKeyValue): int
    {
        die(__METHOD__);
        // TODO: Implement removeByPrimaryKey() method.
    }

    public function loadRowByPrimaryKey($id): ?array
    {
        die(__METHOD__);
        // TODO: Implement loadRowByPrimaryKey() method.
    }

    protected function getConcatCondition(array $condition): string
    {
        die(__METHOD__);
        // TODO: Implement getConcatCondition() method.
    }

    protected function getMany2manyQuery(Many2manyField $field): string
    {
        die(__METHOD__);
        // TODO: Implement getMany2manyColumnName() method.
    }
}
