<?php

class MysqlProxy extends SqlProxy
{
    /**
     * @param bool $isAllColumns
     * @return array
     * @throws StoreException
     * @throws SystemException
     */
    public function loadListValues(bool $isAllColumns = false): array
    {
        $columns = $this->getQueryColumns($isAllColumns);
        $joins   = $this->getQueryJoins($columns);
        $where   = $this->getQueryWhere();

        $having  = $this->getQueryHaving();
        $groupby = $this->getQueryGroupBy();

        $columnsSection = join(', ', $columns);
        $fromSection    = join(' ', $joins);
    
        $sql = "SELECT SQL_CALC_FOUND_ROWS ".$columns[0].", ".$columnsSection.
                " FROM ".$fromSection;
                
        if ($where) {
            $sql .= " WHERE ".join(' AND ', $where);
        }
        
        if ($groupby) {
            $sql .= $groupby;
        }
        
        if ($having) {
            $sql .= " HAVING ".join(' AND ', $having);
        }
        
        $orderBy = $this->getQueryOrderDirection();
        if ($orderBy) {
            $sql .= " ORDER BY ".$orderBy;
        }
        
        if ($this->isUseLimit()) {
            $limit = $this->getQueryLimit();
            $sql .= " LIMIT ".$limit;
        }

        $rows = $this->connection->getAll($sql);

        $total = (int) $this->connection->getOne('SELECT FOUND_ROWS()');
        $this->setCount($total);

        return $rows;
    } // end loadListValues

     public function removeByPrimaryKey(string $primaryKeyValue): int
     {
        $sql = 'DELETE FROM '.$this->store->getName().
               ' WHERE '.$this->store->getPrimaryKey(). ' = '.
               $this->connection->quote($primaryKeyValue).' LIMIT 1';
               
        return $this->connection->query($sql);
     } // end removeByPrimaryKey
    
    public function createAuditTable(string $auditTableName, string $originalTableName): bool
    {
        try {
            $sql = "SELECT 1 FROM ".$auditTableName." LIMIT 1";
            $this->connection->getRow($sql);
        } catch (\Exception $exp) {
            $sql = "CREATE TABLE ".$auditTableName." LIKE ".$originalTableName;

            $this->connection->query($sql);

            $sql = "ALTER TABLE ".$auditTableName." DROP COLUMN id;";

            $this->connection->query($sql);

            $sql = "ALTER TABLE ".$auditTableName."
                    ADD __id int(11) unsigned NOT NULL AUTO_INCREMENT FIRST,
                    ADD __action varchar(100) DEFAULT NULL AFTER __id,
                    ADD __id_author int(11) unsigned DEFAULT NULL AFTER __action,
                    ADD __action_date datetime AFTER __id_author,
                    ADD id int(11) unsigned NOT NULL AFTER __action_date,
                    ADD PRIMARY KEY (__id);";

            $this->connection->query($sql);

            $this->_dropIndexes($auditTableName);
        }

        return true;
    } // end createAuditTable
    
    /**
     * @param Many2manyField $field
     * @return string
     * @throws SystemException
     */
    protected function getMany2manyQuery(Many2manyField $field): string
    {
        $foreignTable = $this->connection->quoteTableName(
            $field->getForeignTable()
        );
        $foreignValueField = $field->getForeignFieldValue();
        
        if (static::isExpression($foreignValueField)) {
            $valuesColumn = $foreignValueField;
        } else {
            $valuesColumn = $foreignTable.static::STORE_NAME_AND_COLUMN_SEPARATOR.$foreignValueField;
        }
        
        $where = $this->getMany2manyWhere($field, $valuesColumn);
    
        if ($field->isUniqueValues()) {
            $valuesColumn = 'DISTINCT '.$valuesColumn;
        }
    
        $linkTable = $field->getLinkTable();
        
        $sql = 'SELECT 
                    GROUP_CONCAT('.$valuesColumn." SEPARATOR '::')".
              ' FROM '.
                    $foreignTable.
              ' JOIN '.$linkTable.
                  ' ON ('.$foreignTable.'.'.$field->getForeignFieldKey().
                  ' = '.$linkTable.'.'.$field->getLinkForeignField().') 
                WHERE '.
                    $linkTable.'.'.$field->getLinkField().' = '.
                    $this->store->getName().'.'.$this->store->getPrimaryKey().
                    $where;
        
        return $sql;
    } // end getMany2manyQuery
    
    private function _dropIndexes(string $tableName): void
    {
        $search = array(
            'Key_name&<>' => 'PRIMARY'
        );

        $where = $this->connection->getSqlCondition($search);
        $sql = 'SHOW INDEX FROM '.$tableName.' WHERE '.join(' AND ', $where);
        $keys = $this->connection->getAll($sql);
        if (!$keys) {
            return;
        }

        foreach ($keys as $key) {
            $sql = 'ALTER TABLE `'.$tableName.
                '` DROP INDEX '.$key['Key_name'].';';

            $this->connection->query($sql);
        }
    }
}
