<?php

class ListActionApiView extends AbstractActionApiView
{
    /**
     * @param AbstractAction $action
     * @param Response $response
     * @param array|null $vars
     * @return bool
     */
    public function onResponse(
        AbstractAction &$action, Response &$response, ?array &$vars
    ): bool
    {
        $response->setType(Response::JSON);

        if (!array_key_exists('tableData', $vars)) {
            $this->getErrorMessage($action, $response, $vars);
            return true;
        }

        $this->getSuccessMessage($action, $response, $vars);

        return true;
    } // end onResponse

    public function getSuccessMessage(
        AbstractAction &$action, Response &$response, &$vars
    )
    {
        $store = $action->getStore();

        $name = $action->getStore()->getName();

        $response->status      = static::STATUS_OK;
        $response->$name       = $vars['tableData'];
        $response->rowsPerPage = $action->getStore()->getRowsPerPageCount();
        $response->totalRows   = $action->getStore()->getTotalCount();
        $response->currentPage = $action->getStore()->getCurrentPageIndex();
        $response->baseUrl     = $action->getUrl();
        $response->pagingMode  = $action->getStore()->getModel()->get('paging');

        // FIXME:
        $response->pageRequestParam = $store->createRequestKey(Store::PAGE_INDEX_KEY_IN_REQUEST);

        $params = array(
            $response->pageRequestParam => '%page%'
        );

        $response->pagerUrl = urldecode(Core::getInstance()->getUrl($response->baseUrl, $params));

        return true;
    } // end getSuccessMessage

    public function getErrorMessage(
        AbstractAction &$action, Response &$response, &$vars)
    {
        $response->setStatus(Response::STATUS_BAD_REQUEST);
        $response->status  = static::STATUS_ERROR;
        $response->message = __(
            "Undefined tableData into ListActionView data"
        );
        return true;
    } // end getErrorMessage
}