<?php

abstract class ValuesObject extends Entity implements JsonSerializable
{
    private $_values;

    public function __construct(array $data = array())
    {
        $this->_values = $data;
    } // end __construct

    protected function get(string $key): mixed
    {
        if ($this->has($key)) {
            return $this->_values[$key];
        }

        throw new SystemException('Undefined property: '.$key);
    } // end get

    protected function has($key): bool
    {
        return array_key_exists($key, $this->_values);
    }

    public function toJson(): string
    {
        return json_encode($this->_values);
    }
    
    protected function set(string $key, mixed $value): void
    {
        $this->_values[$key] = $value;
    }

    /**
     * Returns values in array.
     *
     * @return array
     */
    public function getValues(): array
    {
        return $this->_values;
    } // end getValues

    public function isEmpty(): bool
    {
        return empty($this->_values);
    } // end isEmpty

    public static function convert(array $rows): mixed
    {
        $className = get_called_class();
    
        if (is_scalar($rows)) {
            return new $className($rows);
        }
    
        $result = array();
        foreach ($rows as $key => $values) {
            $result[$key] = new $className($values);
        }
    
        return $result;
    } // end convert
    
    /**
     * @return mixed
     * @todo change return type to mixed
     */
    public function jsonSerialize()
    {
        return $this->_values;
    } // end jsonSerialize
}