<?php

namespace core\util;

use FestiUtils;
use ReflectionClass;
use ValuesObject;

class ValuesObjectMapper
{
    const SELECTOR_SEPARATOR = '__';

    private $_filters = array();

    public function map(array $data, string $className): ValuesObject
    {
        return $this->_getConvertedArray($data, $className);
    } // end map

    private function _getConvertedArray(array $data, string $className, string $selectorPrefix = ''): ValuesObject
    {
        $classReflection = new ReflectionClass($className);

        $classInstance = new $className(array());

        foreach ($data as $key => $value) {

            $res = $this->_applyFilter($classInstance, $key, $value, $selectorPrefix);
            if ($res === true) {
                continue;
            }

            $methodPostfix = FestiUtils::convertToCamelCase($key);
            $methodName = 'set'.$methodPostfix;

            if (!$classReflection->hasMethod($methodName)) {
                continue;
            }

            if (is_array($value)) {
                $parentSelectorPrefix = $selectorPrefix.$key.static::SELECTOR_SEPARATOR;
                $value = $this->_getArrayItem($classReflection, $methodName, $value, $parentSelectorPrefix);
            }

            $classInstance->$methodName($value);
        }

        return $classInstance;
    } // end _getConvertedArray

    private function _applyFilter(ValuesObject &$valuesObject, string $key, $value, string $selectorPrefix): bool
    {
        $filterKey = $selectorPrefix.$key;

        if (!array_key_exists($filterKey, $this->_filters)) {
            return false;
        }

        $params = array(
            &$valuesObject,
            $key,
            $value
        );

        return call_user_func_array($this->_filters[$filterKey], $params);
    } // end _applyFilter

    private function _getArrayItem(
        ReflectionClass $classReflection, string $methodName, array $values, string $parentSelectorPrefix
    ): ?array
    {
        $method = $classReflection->getMethod($methodName);

        $comment = $method->getDocComment();

        $regExp = "#@param (?<class>[^\[]+)\[\]#Umis";
        if (!preg_match($regExp, $comment, $matches)) {
            return $values;
        }

        if (!class_exists($matches['class'])) {
            return null;
        }

        $result = array();
        foreach ($values as $key => $row) {
            $result[$key] = $this->_getConvertedArray($row, $matches['class'], $parentSelectorPrefix);
        }

        return $result;
    } // end _getArrayItem

    /**
     * @param string $selector
     * @param callable $callback
     */
    public function addFilter(string $selector, Callable $callback): void
    {
        $this->_filters[$selector] = $callback;
    }

}