<?php

use cron\AlreadyRunningCronWorkerException;
use cron\CronQueueSingletonWorker;
use cron\CronSingletonWorker;
use cron\CronWorkerManager;

class Bundle_Cron_CronSingletonWorkerTest extends FestiTestCase
{
    const DEFAULT_QUEUE_NAME = 'queue_simple';

    public function testSimpleSingletonWorker()
    {
        $worker = new SimpleSingletonWorker();

        $worker->start();

        $hasException = false;
        try {
            $secondWorker = new SimpleSingletonWorker();
        } catch (AlreadyRunningCronWorkerException $exp) {
            $hasException = true;
        }

        $this->assertTrue($hasException);
    } // end testSimpleSingletonWorker

    public function testSimpleQueueSingletonWorker()
    {
        $connection =  $this->getDbConnection();

        $options = array(
            'id' => 1,
            'limit' => 10
        );

        $worker = new SimpleQueueSingletonWorker($connection, $options);

        $hasException = false;
        try {
            $secondWorker = new SimpleSingletonWorker();
        } catch (AlreadyRunningCronWorkerException $exp) {
            $hasException = true;
        }

        $this->assertTrue($hasException);

        $data = array(
            'status' => true
        );

        $this->_insertQueueRow($data, static::DEFAULT_QUEUE_NAME);

        $worker->start();

        $result = $worker->getLastResult();

        $this->assertTrue($result['status'] == $data['status']);

        $sql = "SELECT * FROM ".static::DEFAULT_QUEUE_NAME;
        $rows = $connection->getAll($sql);
        $this->assertEmpty($rows);

    } // end testSimpleSingletonWorker

    private function _insertQueueRow($data, $queueName)
    {
        $connection =  $this->getDbConnection();

        $values = array(
            'data' => json_encode($data)
        );

        $connection->insert($queueName, $values);
    }

    public function testSimpleCronManager()
    {
        $connection = $this->getDbConnection();

        //
        $values = array(
            array('data' => FS_RESOURCES_ROOT.'/cron_worker.php --name common --id 1'),
            array('data' => FS_RESOURCES_ROOT.'/cron_worker.php --name common --id 2')
        );

        $connection->massInsert("cron_workers", $values);
        //

        $manager = new SimpleCronManager($connection);

        $manager->start();

        $scripts = $manager->getExecutedScripts();

        $this->assertTrue(count($scripts) == 2);
    }

}

class SimpleQueueSingletonWorker extends CronQueueSingletonWorker
{

    private $_iteration = 0;
    private $_lastResult;

    public function getStorageName(): string
    {
        return Bundle_Cron_CronSingletonWorkerTest::DEFAULT_QUEUE_NAME;
    }

    protected function hasNextIteration(): bool
    {
        if ($this->_iteration > 1) {
            return false;
        }

        $this->_iteration++;
        return true;
    }

    protected function onRow($record)
    {
        $this->_lastResult = json_decode($record['data'], true);
    }

    public function getLastResult()
    {
        return $this->_lastResult;
    }
}


class SimpleCronManager extends CronWorkerManager
{
    private $_executedScripts = array();
    private $_iteration = 0;

    protected function onRow($app)
    {
        $this->_executedScripts[] = $app['data'];
        $this->startProcess($app['data']);
    } // end onRow

    public function getExecutedScripts()
    {
        return $this->_executedScripts;
    } // end getExecutedScripts

    protected function hasNextIteration(): bool
    {
        if ($this->_iteration > 1) {
            return false;
        }

        $this->_iteration++;
        return true;
    }

    /**
     * @override
     */
    protected function onDelay()
    {
    } // end onDelay

    protected function getOptions()
    {
        return array();
    }

    public function getWorkerColumnNameInStorage(): string
    {
        return 'status';
    }

    /**
     * @override
     */
    public function getWorkerSteadyValue(): string
    {
        return 'steady';
    } // end getWorkerValue

    /**
     * @override
     */
    protected function getBaseLockPath()
    {
        return FS_RESOURCES_ROOT.'locks/';
    } // end getBaseLockPath

}

class SimpleSingletonWorker extends CronSingletonWorker
{
    public function start()
    {
    }
}