<?php

class EventDispatcherTest extends FestiTestCase
{
    const EXCEPTION_MESSAGE = 'Not Found Listener, Type EVENT_UPDATE_VALUES';
    
    private $_store;
    private $_anonymousFunction;

    public function testOnAddEventListener()
    {
        $this->_prepareStore();

        $listeners = $this->_store->getEventListeners();
        
        $this->assertTrue(
            array_key_exists(Store::EVENT_UPDATE_VALUES, $listeners)
        );
        
        $this->assertTrue(count($listeners[Store::EVENT_UPDATE_VALUES]) == 2);
        
        return true;
    }

    public function testOnRemoveEventListener()
    {
        $this->testOnAddEventListener();

        $this->_store->removeEventListener(
            Store::EVENT_UPDATE_VALUES,
            $this->_anonymousFunction
        );
        
        $listeners = $this->_store->getEventListeners();
        
        $this->assertTrue(
            array_key_exists(Store::EVENT_UPDATE_VALUES, $listeners)
        );

        $this->assertTrue(count($listeners[Store::EVENT_UPDATE_VALUES]) == 1);

        return true;
    }

    public function testOnRemoveEventListenerNotFoundType()
    {
        $this->_store = $this->getDefaultStore();

        try {
            $this->_store->removeEventListener(
                Store::EVENT_UPDATE_VALUES,
                function ($event) {
                    throw new SystemException("One Closure");
                }
            );
            $this->fail("Expected SystemException Not Thrown");
        } catch (SystemException $exp) {
            $regexp = sprintf('/%s/', static::EXCEPTION_MESSAGE);
            $this->assertMatchesRegularExpression($regexp, $exp->getMessage());
        }

        return true;
    }
     
    private function _prepareStore()
    {
        $this->_store = $this->getDefaultStore();
        
        $this->_store->addEventListener(
            Store::EVENT_UPDATE_VALUES,
            function ($event) {
                throw new SystemException("First Closure");
            }
        );

        $this->_setAnonymousFunction();
        
        $this->_store->addEventListener(
            Store::EVENT_UPDATE_VALUES,
            $this->_anonymousFunction
        );

        return true;
    }
    
    private function _setAnonymousFunction()
    {
        $this->_anonymousFunction = function ($event) {
            throw new SystemException("Second Closure");
        };
        
        return true;
    }

    public function testSomeListenerInstances()
    {
        $this->_store = $this->getDefaultStore();

        $this->_doCheckListeners($this->_store, 1);

        $otherStore = $this->getDefaultStore();

        $this->assertFalse($this->_store === $otherStore);
        $this->assertEquals(get_class($this->_store), get_class($otherStore));

        $this->_doCheckListeners($otherStore, 2);

        return true;
    }

    private function _doCheckListeners(&$instance, $expectedCount)
    {
        $this->_store->addEventListener(
            Store::EVENT_STORE_INIT,
            array(&$instance, 'getName')
        );

        $listeners = $this->_store->getEventListeners();

        $this->assertTrue(
            array_key_exists(Store::EVENT_STORE_INIT, $listeners)
        );

        $countListeners = count($listeners[Store::EVENT_STORE_INIT]);

        $this->assertEquals($expectedCount, $countListeners);
    }
}
