<?php

class LocaleTest extends FestiTestCase
{
    /**
     * @var LocaleModel
     */
    private $_defaultLocaleModel;

    protected function setUp(): void
    {
        parent::setUp();

        $core = Core::getInstance();

        $this->_defaultLocaleModel = $core->getLocaleModel();

        $file = FS_RESOURCES_ROOT.'locale'.DIRECTORY_SEPARATOR.'ru.mo';
        $dictionary = new MoDictionaryLocale($file);
        $testLocaleModel = new LocaleModel($dictionary);

        $core->setLocaleModel($testLocaleModel);
    }

    protected function tearDown(): void
    {
        parent::tearDown();

        Core::getInstance()->setLocaleModel($this->_defaultLocaleModel);
    }

    public function testTranslate(): void
    {
        $field = 'caption';

        $message   = '%s is required field';
        $translate = "{$field} - обязательное поле";

        $this->assertEquals($translate, __($message, $field));
    }

    public function testPluralTranslate(): void
    {
        $translates  = $this->_getTranslates();
        $pluralForms = $this->_getPluralForms();

        foreach ($pluralForms as $count => $pluralForm) {
            $expected = sprintf($translates[$pluralForm], $count);
            $actual   = __p('%s hour ago', '%s hours ago', $count, $count);

            $this->assertEquals($expected, $actual);
        }
    }

    public function testOriginalTranslates(): void
    {
        $messageSingular = 'One untranslated thing';
        $messagePlural   = 'Many untranslated things';

        $this->assertEquals($messageSingular, __($messageSingular));

        $singularCounts = array(1, -1);
        foreach ($singularCounts as $count) {
            $translated = __p($messageSingular, $messagePlural, $count);
            $this->assertEquals($messageSingular, $translated);
        }

        $pluralCounts = array(-2, 0, 2);
        foreach ($pluralCounts as $count) {
            $translated = __p($messageSingular, $messagePlural, $count);
            $this->assertEquals($messagePlural, $translated);
        }
    }

    /**
     * @return string[] - array($pluralForm => $translate)
     */
    private function _getTranslates(): array
    {
        return array(
            '%s час назад',
            '%s часа назад',
            '%s часов назад',
        );
    }

    /**
     * @return int[] - array($count => $pluralForm)
     */
    private function _getPluralForms(): array
    {
        return array(
            0   => 2,
            1   => 0,
            2   => 1,
            3   => 1,
            4   => 1,
            5   => 2,
            6   => 2,
            7   => 2,
            8   => 2,
            9   => 2,
            10  => 2,
            11  => 2,
            12  => 2,
            13  => 2,
            14  => 2,
            15  => 2,
            16  => 2,
            17  => 2,
            18  => 2,
            19  => 2,
            20  => 2,
            21  => 0,
            22  => 1,
            23  => 1,
            24  => 1,
            25  => 2,
            26  => 2,
            99  => 2,
            100 => 2,
            101 => 0,
            102 => 1,
            -1  => 0,
            -2  => 1,
            -3  => 1,
            -4  => 1,
            -5  => 2,
            -11 => 2,
        );
    }
}
