<?php

use tests\Resources\store\FestiSettingsStore;

class StageBuilderTest extends FestiTestCase
{
    public function testContentBlock(): void
    {
        $stage = new StageBuilder(1, StageBuilder::LAYOUT_ROWS);

        $blockContent = 'test stage content block '.time();
        $stage->addBlock($blockContent);

        $response = new Response();
        $stage->onRequest($response);

        $this->assertMatchesRegularExpression("#".$blockContent."#Umis", $response->content);
    }

    public function testCallbackBlock(): void
    {
        $stage = new StageBuilder(1, StageBuilder::LAYOUT_ROWS);

        $blockContent = 'test stage callback block '.time();
        $params = array(
            'value',
        );

        $stage->addCallbackBlock(
            function (Response &$response, $param) use ($blockContent) {

                $this->assertEquals('value', $param);

                $response->content = $blockContent;

                return true;
            },
            $params
        );

        $response = new Response();
        $stage->onRequest($response);

        $this->assertMatchesRegularExpression("#".$blockContent."#Umis", $response->content);
    }

    public function testStoreBlock(): void
    {
        $stage = new StageBuilder(1, StageBuilder::LAYOUT_ROWS);

        $store = Core::getInstance()->createStoreInstance('settings');

        $stage->addStoreBlock($store);

        $response = new Response();
        $stage->onRequest($response);

        $this->assertMatchesRegularExpression("#b-db-section-settings#Umis", $response->content);
        $this->assertMatchesRegularExpression("#id=\"dgs-content-settings\"#Umis", $response->content);
    }

    public function testStoreActionBlock(): void
    {
        $stage = new StageBuilder(1, StageBuilder::LAYOUT_ROWS);

        $store = Core::getInstance()->createStoreInstance('settings');
        $action = $store->createActionInstance(Store::ACTION_INSERT);

        $stage->addStoreActionBlock($action);

        $response = new Response();
        $stage->onRequest($response);

        $this->assertMatchesRegularExpression("#b-db-form-settings#Umis", $response->content);
        $this->assertMatchesRegularExpression("#f-db-form#Umis", $response->content);
    }

    public function testCustomStoreClassBlock(): void
    {
        $store = new FestiSettingsStore();

        $stage = new StageBuilder(1, StageBuilder::LAYOUT_ROWS);
        $stage->addStoreBlock($store);

        $blockContent = 'test stage content with store block '.time();
        $stage->addBlock($blockContent);

        $response = new Response();
        $stage->onRequest($response);

        $this->assertMatchesRegularExpression("#id=\"dgs-content-settings\"#Umis", $response->content);

        $this->assertMatchesRegularExpression("#".$blockContent."#Umis", $response->content);
    }
}
