<?php

namespace tests\Bundle\Store\Action;

use FestiTestCase;
use Response;
use Store;

class BatchInsertActionTest extends FestiTestCase
{
    public function testInsert(): void
    {
        $core = $this->getCoreWithConnection();

        $store = $core->createStoreInstance(STORE_DEFAULT);
        $action = $store->createActionInstance(Store::ACTION_BATCH_INSERT);

        $this->_testBatchInsert($store, $action);
        $this->_testBatchInsertWithError($store, $action);
    }
    
    private function _testBatchInsert(&$store, &$action): void
    {
        $values = array(
            array(
                'caption' => 'Test',
                'value' => 'Value Test'
            ),
            array(
                'caption' => 'Test_Two',
                'value' => 'Value Test2'
            ),
        );
        
        $response = new Response();
        
        $_REQUEST = $values; 
        
        $action->onUpdate($response);
        
        $data = $store->load();

        $this->assertTrue($data[0]['value'] == $values[0]['value']);
        $this->assertTrue($data[1]['value'] == $values[1]['value']);
        $this->assertEmpty($action->getLastError());
        
        $this->_removeData($store, $data);
    }
    
    private function _testBatchInsertWithError(&$store, &$action): void
    {
        $values = array(
            array(
                'value' => 'Value Test'
            ),
            array(
                'caption' => 'Test_Two',
                'value' => 'Value Test2'
            ),
        );
        
        $response = new Response();
        
        $_REQUEST = $values; 
        
        $action->onUpdate($response);
            
        $errorMessage = $action->getLastError();
        
        $this->assertTrue($errorMessage == "Caption is required field");
        
        $data = $store->load();

        $this->assertEmpty($data);
        
        $this->_removeData($store, $data);
    }
    
    private function _removeData(&$store, $data): void
    {
        foreach ($data as $item) {
            $store->removeByPrimaryKey($item['id']);
        }
    }
    
    public function testOnStartApi(): void
    {
        $core = $this->getCoreWithConnection();

        $store = $core->createStoreInstance('settings_api');


        $this->_testBatchInsertApi($store);
        $this->_testBatchInsertApiWithError($store);
    }

    private function _testBatchInsertApi(&$store): void
    {
        $action = $store->createActionInstance(Store::ACTION_BATCH_INSERT);

        $values = array(
            array(
                'caption' => 'Test',
                'value'   => 'Value Test'
            ),
            array(
                'caption' => 'Test_Two',
                'value'   => 'Value Test2'
            ),
        );
        
        $response = new Response();
        
        $stream = &$store->getModel()->getStream();
        
        $filePath = FS_RESOURCES_ROOT.'data.json';
        
        $isCreate = file_put_contents($filePath, json_encode($values));
        $this->assertTrue((bool) $isCreate);
        
        $stream = $filePath;
        
        $action->onStart($response);
        
        $this->assertTrue((bool) unlink($filePath));
        
        $data = $store->load();
        
        $this->assertTrue($data[0]['value'] == $values[0]['value']);
        $this->assertTrue($data[1]['value'] == $values[1]['value']);
        $this->assertEmpty($action->getLastError());
        
        $this->_removeData($store, $data);
    }
    
    private function _testBatchInsertApiWithError($store): void
    {
        $action = $store->createActionInstance(Store::ACTION_BATCH_INSERT);

        $values = array(
            array(
                'value'   => 'Value Test'
            ),
            array(
                'caption' => 'Test_Two',
                'value'   => 'Value Test2'
            ),
        );

        $response = new Response();

        $stream = &$store->getModel()->getStream();

        $filePath = FS_RESOURCES_ROOT.'data.json';

        $isCreate = file_put_contents($filePath, json_encode($values));
        $this->assertTrue((bool) $isCreate);
        
        $stream = $filePath;
        
        $action->onUpdate($response);
        
        $this->assertTrue((bool) unlink($filePath));
        
        $errorMessage = $action->getLastError();

        $this->assertTrue($errorMessage == "Caption is required field");

        $data = $store->load();

        $this->assertEmpty($data);

        $this->_removeData($store, $data);
    }
}
