<?php

namespace tests\Bundle\Store\Action;

use FestiTestCase;
use Response;
use Store;
use Throwable;

class CsvImportActionTest extends FestiTestCase
{
    private const TEST_IMPORT_DIR = FS_RESOURCES_ROOT.'import'.DIRECTORY_SEPARATOR.
        'test'.DIRECTORY_SEPARATOR;

    public function testDoImportCreateFolder(): void
    {
        $store = $this->core->createStoreInstance('settings');

        $actions = &$store->getModel()->getActions();

        $actions[Store::ACTION_CSV_IMPORT]['uploadDirPath'] = static::TEST_IMPORT_DIR;

        $response = new Response();

        $csvImportAction = $store->createActionInstance(Store::ACTION_CSV_IMPORT);
        $this->preparePostForStore(Store::ACTION_CSV_IMPORT, array(), $store);

        $_FILES = array(
            $csvImportAction->getActionName() => array(
                'name'     => "settings.csv",
                'type'     => "application/vnd.ms-excel",
                'tmp_name' => "/var/www/user/data/bin-tmp/phpGyX8Zg",
                'error'    => 0,
                'size'     => 45,
            )
        );

        try {
            $csvImportAction->onStart($response);
            $this->fail("Expect SystemException");
        } catch (Throwable $exp) {
            $this->assertEquals(
                $exp->getMessage(), __('File Not Download')
            );
        }

        $this->assertTrue(is_dir(static::TEST_IMPORT_DIR));

        rmdir(static::TEST_IMPORT_DIR);
    }

    public function testDoImport(): void
    {   
        $store = $this->getDefaultStore();
        
        $mock = $this->getActionMockBuilder(
            'CsvImport',
            array('getUploadFilePath')
        );
        
        $mock->method('getUploadFilePath')->willReturn(
            FS_RESOURCES_ROOT.'import'.DIRECTORY_SEPARATOR.'settings.csv'
        );
        
        $response = new Response();
        
        $this->preparePostForStore(Store::ACTION_CSV_IMPORT, array(), $store);
       
        $mock->onStart($response);
        
        $testValues = array(
            array(
                'caption' => 'testImport1',
                'value'   => 'testValue1'
            ),
            array(
                'caption' => 'testImport2',
                'value'   => 'testValue2'
            )
        );
        $data = $store->load();
        
        $this->assertTrue($data[0]['value'] == $testValues[0]['value']);
        $this->assertTrue($data[1]['value'] == $testValues[1]['value']);
        
        $this->_removeData($store, $data);
    }
    
    private function _removeData(&$store, $data): void
    {
        foreach ($data as $item) {
            $store->removeByPrimaryKey($item['id']);
        }
    }
}
