<?php

namespace tests\Bundle\Store\Action;

use EditAction;
use FestiTestCase;
use Response;
use Store;

class EditActionTest extends FestiTestCase
{
    private $_primaryKeyValue;
    
    public function testOnStart(): void
    {
        $store = $this->getDefaultStore();
        
        $this->_testInsertRequest($store);
        
        $action = $store->createActionInstance(Store::ACTION_EDIT);
        $this->assertInstanceOf(EditAction::class, $action);
        
        $this->_testEditForm($store, $action);
        $this->_testEditRequest($store, $action);
    
    } // end testGetInstanceWithoutOptions
    
    private function _testEditForm(&$store, &$action): void
    {
        FestiTestCase::preparePrimaryKey($store, $this->_primaryKeyValue);
        
        $response = new Response();
        $action->onStart($response);
        
        $this->assertMatchesRegularExpression(
            "#f-db-form#Umis", 
            $response->content, 
            "Not found insert form."
        );
    } // end _testInsertForm
    
    private function _testInsertRequest(&$store): void
    {
        $values = array(
            'caption' => 'Test',
            'value'   => 'test'
        );
        
        FestiTestCase::preparePostForStore(
            Store::ACTION_INSERT, 
            $values, 
            $store
        );
        
        $action = $store->createActionInstance(Store::ACTION_INSERT);
        
        $response = new Response();
        $action->onStart($response);
        
        $data = $store->load();
        
        $this->assertTrue(count($data) == 1);
        $this->assertTrue($data[0]['value'] == $values['value']);
        
        $this->_primaryKeyValue = $data[0]['id'];
    } // end _testInsertRequest
    
    private function _testEditRequest(&$store, &$action): void
    {    
        $values = array(
            'caption' => 'Test',
            'value'   => 'test2'
        );
        
        FestiTestCase::preparePostForStore(
            Store::ACTION_EDIT, 
            $values, 
            $store, 
            $this->_primaryKeyValue
        );
        
        $response = new Response();
        $action->onStart($response);
        
        $data = $store->load();
        
        $this->assertTrue(count($data) == 1);
        $this->assertTrue($data[0]['value'] == $values['value']);
    } // end _testInsertRequest
}