<?php

namespace tests\Bundle\Store\Action;

use FestiTestCase;
use InsertAction;
use Response;
use Store;

class GridEditorActionTest extends FestiTestCase
{
    public function testOnStart(): void
    {
        $store = $this->getDefaultStore();

        $action = $store->createActionInstance(Store::ACTION_INSERT);

        $this->assertInstanceOf(InsertAction::class, $action);

        $this->_testInsertRequest($store, $action);

        $this->_testGridEditor($store);
    } // end testOnStart

    private function _testInsertRequest(&$store): void
    {
        $values = array(
            'caption' => 'Test',
            'value'   => 'test'
        );

        FestiTestCase::preparePostForStore(
            Store::ACTION_INSERT,
            $values,
            $store
        );

        $action = $store->createActionInstance(Store::ACTION_INSERT);

        $response = new Response();
        $action->onStart($response);

        $data = $store->load();

        $this->assertTrue(count($data) == 1);
        $this->assertTrue($data[0]['value'] == $values['value']);

        $this->_primaryKeyValue = $data[0]['id'];

        $this->preparePrimaryKey($store, $this->_primaryKeyValue);
        $store->addAllowedTableRow($this->_primaryKeyValue);
    } // end _testInsertRequest

    private function _prepareRequest($storeName, $value = false): void
    {
        $_REQUEST = array(
            $storeName => array(
                'action' => Store::ACTION_GRID_EDITOR,
                'ID'     => $this->_primaryKeyValue,
                'field' => 'caption'
            ),
        );

        if ($value) {
            $_REQUEST['caption'] = $value;
        }
    }

    private function _testGridEditor(&$store): void
    {
        $storeName = $store->getIdent();

        $action = $store->createActionInstance(Store::ACTION_GRID_EDITOR);

        $this->_prepareRequest($storeName);

        $response = new Response();
        $action->onStart($response);

        $this->assertNotEmpty($response->content);

        $newValue = 'new_test';
        $this->_prepareRequest($storeName, $newValue);

        $_POST[$storeName]['performPost'] = 'save';

        $response = new Response();
        $action->onStart($response);

        $data = $store->load();

        $this->assertTrue(count($data) == 1);
        $this->assertTrue($data[0]['caption'] == $newValue);
    } // end _testGridEditor
}
