<?php

namespace tests\Bundle\Store\Action;

use FestiTestCase;
use InfoAction;
use InsertAction;
use Response;
use Store;

class InfoActionApiTest extends FestiTestCase
{
    private $_primaryKeyValue;
    
    public function testOnStartApi(): void
    {
        $values = array(
            'caption' => 'Test',
            'value'   => 'test'
        );
        
        $this->_testInsertRequest($values);
        
        $this->_testInfoApi($values);
    }
    
    private function _testInsertRequest($values): void
    {
        $store = $this->getDefaultStore();
        
        $action = $store->createActionInstance(Store::ACTION_INSERT);
        
        $this->assertInstanceOf(InsertAction::class, $action);
    
        FestiTestCase::preparePostForStore(
            Store::ACTION_INSERT,
            $values,
            $store
        );
    
        $response = new Response();
    
        $action->onStart($response);
    
        $data = $store->load();
    
        $this->assertTrue(count($data) == 1);
        $this->assertTrue($data[0]['value'] == $values['value']);
        
        $this->_primaryKeyValue = $data[0]['id'];
    } // end _testInsertRequest
    
    private function _testInfoApi($values): void
    {
        $core = $this->getCoreWithConnection();
        
        $store = $core->createStoreInstance('settings_api');
        
        $action = $store->createActionInstance(Store::ACTION_INFO);
        
        $this->assertInstanceOf(InfoAction::class, $action);
        
        FestiTestCase::preparePostForStore(
            Store::ACTION_INFO,
            array(),
            $store,
            $this->_primaryKeyValue
        );
        
        $response = new Response();
        
        $action->onStart($response);
        
        $this->assertNotNull($response);
        
        $responseValues = $response->getArrayCopy();
        
        $this->assertEquals($values['value'], $responseValues['value']);
        $this->assertEquals($values['caption'], $responseValues['caption']);
        
        $this->assertEmpty($action->getLastError());
        
        $store->removeByPrimaryKey($this->_primaryKeyValue);
    }
}
