<?php

namespace tests\Bundle\Store\Action;

use FestiTestCase;
use InfoAction;
use Response;
use stdClass;
use Store;
use Throwable;

class InfoActionTest extends FestiTestCase
{
    private $_primaryKeyValue;

    public function testOnStart(): void
    {
        $core = $this->getCoreWithConnection();
        
        $store = $core->createStoreInstance(STORE_DEFAULT);
        
        $this->_testInsertRequest($store);
        
        $this->_testInfoForm($store);
        
        $this->_testInfoFormWithAlert($store);
    } // end testOnStart
    
    private function _testInsertRequest(&$store): void
    {
        $values = array(
            'caption' => 'Test',
            'value'   => 'test'
        );
        
        FestiTestCase::preparePostForStore(
            Store::ACTION_INSERT, 
            $values, 
            $store
        );
        
        $action = $store->createActionInstance(Store::ACTION_INSERT);
        
        $response = new Response();
        $action->onStart($response);
        
        $data = $store->load();
        
        $this->assertTrue(count($data) == 1);
        $this->assertTrue($data[0]['value'] == $values['value']);
        
        $this->_primaryKeyValue = $data[0]['id'];
    } // end _testInsertRequest
    
    private function _testInfoForm(&$store): void
    {   
        $action = $store->createActionInstance(Store::ACTION_INFO);
        
        $this->assertInstanceOf(InfoAction::class, $action);
        
        FestiTestCase::preparePostForStore(
            Store::ACTION_INFO, 
            array(), 
            $store, 
            $this->_primaryKeyValue
        );

        $response = new Response();
        
        $action->onStart($response);

        ob_start();
        
        $response->send();
        $content = ob_get_contents();
        
        ob_end_clean();
        
        $this->assertMatchesRegularExpression(
            "#Test#Umis", 
            $response->content, 
            "Not found Info form."
        );
    } // end _testInfoForm
    
    private function _testInfoFormWithAlert(&$store): void
    {   
        $action = $store->createActionInstance(Store::ACTION_INFO);
        
        $this->assertInstanceOf(InfoAction::class, $action);
        
        FestiTestCase::preparePostForStore(
            Store::ACTION_INFO, 
            array(), 
            $store,
            new stdClass() //XXX: By PDOException
        );

        $response = new Response();

        $isGetException = false;
        try {
            $action->onStart($response);
        } catch (Throwable $exp) {
            $isGetException = true;
        }

        $this->assertTrue($isGetException);

        /*
        ob_start();
        
        $response->send();
        $content = ob_get_contents();
        
        ob_end_clean();

        $this->assertMatchesRegularExpression(
            "#\"type\":\"alert\"#Umis",
            $content, 
            "Must be Alert Type"
        );
        */
    } // end _testInfoFormWithAlert
    
    public function testDisplayFieldsWithOnlyListAttribute(): void
    {
        $store = $this->getDefaultStore();
        $fields = &$store->getModel()->getFields();
        
        $fields['caption']->set('onlyList', true);
        
        $action = $store->createActionInstance(Store::ACTION_INFO);
        
        $action = $store->createActionInstance(Store::ACTION_INFO);
        
        $this->assertInstanceOf(InfoAction::class, $action);
        
        $response = new Response();
        $action->onStart($response);
        
        $this->assertMatchesRegularExpression(
            "#".$fields['caption']->getCaption()."#Umis",
            $response->getContent(),
            "Not found field with onlyList attribute."
        );
    }
}
