<?php

namespace tests\Bundle\Store\Field;

use FestiTestCase;
use InsertAction;
use Response;
use Store;
use ListAction;

class CompositeFieldTest extends FestiTestCase
{
    public function testOnList(): void
    {
        $core = $this->getCoreWithConnection();

        $store = $core->createStoreInstance('composite_field');
       
        $this->_prepareStore($store);
        
        $action = $store->createActionInstance(Store::ACTION_LIST);

        $this->assertInstanceOf(ListAction::class, $action);

        $response = new Response();

        $action->onStart($response);
        
        $this->assertNotEmpty($response->content);
        $this->assertInstanceOf(ListAction::class, $response->storeAction);
    }
    
    private function _prepareStore(Store $store): void
    {
        $values = array(
            'caption' => 'Caption Test',
            'value'   => 'setting'
        );
    
        FestiTestCase::preparePostForStore(
            Store::ACTION_INSERT,
            $values,
            $store
        );
    
        $response = new Response();
        
        $action = $store->createActionInstance(Store::ACTION_INSERT);

        $this->assertInstanceOf(InsertAction::class, $action);
        
        $action->onStart($response);
    
        $data = $store->load();
        $dbValues = array_shift($data);

        $this->assertArrayHasKey('caption', $dbValues);
    
        $this->assertTrue($dbValues['caption'] == $values['caption']);
    }
}
