<?php

namespace tests\Bundle\Store\Field;

use FestiTestCase;
use InsertAction;
use Response;
use Store;

class SerializeFieldTest extends FestiTestCase
{
    const REQUIRED_ERROR_MESSAGE = 'Value is required field';
    
    public function testOnInsert(): void
    {
        $core  = $this->getCoreWithConnection();
        $store = $core->createStoreInstance('serialize_field');
    
        $this->_testRequired($store);
        
        $this->_testInsert($store);
    }
    
    private function _testRequired($store): void
    {
        $values = $this->_getEmptyValues();
        
        FestiTestCase::preparePostForStore(
            Store::ACTION_INSERT,
            $values,
            $store
        );
    
        $action   = $this->_getInsertAction($store);
        
        $response = new Response();
        
        $action->onStart($response);
        
        $this->assertEquals(
            $action->getLastError(),
            static::REQUIRED_ERROR_MESSAGE
        );
    }
    
    private function _testInsert($store): void
    {
        $values = $this->_getValues();
    
        FestiTestCase::preparePostForStore(
            Store::ACTION_INSERT,
            $values,
            $store
        );
    
        $action   = $this->_getInsertAction($store);
    
        $response = new Response();
    
        $action->onStart($response);
    
        $data = $store->load();
       
        $row  = array_shift($data);
        
        $this->assertTrue((bool) $row);
        $this->assertArrayHasKey('value', $row);
        
        $value = json_decode($row['value'], true);
        $login = array_shift($values['value']['value']);
        
        $this->assertEquals($value['login'], $login);
    }

    private function _getInsertAction($store): InsertAction
    {
        $action = $store->createActionInstance(Store::ACTION_INSERT);
    
        $this->assertInstanceOf(InsertAction::class, $action);
        
        return $action;
    }
    
    private function _getValues(): array
    {
        return array(
            'caption' => 'Test',
            'value'   => array(
                'key' => array(
                    'login',
                    'password',
                ),
                'value' => array(
                    'admin',
                    'password',
                ),
            ),
        );
    }
    
    private function _getEmptyValues(): array
    {
        return array(
            'caption' => 'Test',
            'value'   => array(
                'key' => array(
                    '',
                ),
                'value' => array(
                    '',
                ),
            ),
        );
    }
}
