<?php

namespace tests\Bundle\Store\Field;

use Core;
use DataAccessObject;
use FestiTestCase;
use ListAction;
use Response;
use Store;

class SqlFieldTest extends FestiTestCase
{
    public function testSubQuery(): void
    {
        $core = $this->getCoreWithConnection();

        $values = array(
            array(
                'name' => 'Test'
            ),
            array(
                'name' => 'Test 2'
            )
        );

        $core->db->massInsert('site_contents', $values);

        $options = array();

        $options['model'] = $this->_getDefaultModel();

        $store = Core::getInstance()->createStoreInstance(
            'site_contents',
            $options
        );

        $action = $store->createActionInstance(Store::ACTION_LIST);

        assert($action instanceof ListAction);

        $response = new Response();
        $action->onStart($response);
        $values = $action->getValues();

        $this->assertTrue(count($values) == 2);
        list($row) = $values;
        $this->assertTrue($row['test'] == 2);
    } // end testSubQuery

    public function testForeignKeySql(): void
    {
        $core = $this->getCoreWithConnection();

        $values = array(
            array(
                'id' => 1,
                'caption' => 'Test'
            ),
            array(
                'id' => 2,
                'caption' => 'Test 2'
            )
        );

        if ($core->db->getDatabaseType() === DataAccessObject::TYPE_MSSQL) {
            $core->db->query('SET IDENTITY_INSERT settings ON');
        }

        $core->db->massInsert('settings', $values);

        if ($core->db->getDatabaseType() === DataAccessObject::TYPE_MSSQL) {
            $core->db->query('SET IDENTITY_INSERT settings OFF');
        }

        $values = array(
            'caption' => 'Foreign Sql',
            'is_used' => 1,
            'id_param' => 2
        );

        $core->db->insert("dgs_test_table", $values);
        //

        $store = $core->createStoreInstance('dgs_foreignkey_sql');

        $action = $store->createActionInstance(Store::ACTION_LIST);

        assert($action instanceof ListAction);

        $field = $store->getModel()->getFieldByName('id_param');

        $field->set('foreignValueField', 'CONCAT(settings.caption, \' | \', settings.id)');

        $response = new Response();
        $action->onStart($response);
        $values = $action->getValues();
        list($row) = $values;
        $this->assertTrue($row['id_param'] == "Test 2 | 2");
    }

    private function _getDefaultModel(string $type = 'sql'): array
    {
        return array(
            'table' => array(
                'name'       => 'site_contents',
                'primaryKey' => 'id',
                'exceptionMode' => true
            ),

            'fields' => array(
                'value' => array(
                    'type' => $type,
                    'name' => 'test',
                    'caption' => 'Test',
                    'query' => 'SELECT COUNT(*) FROM site_contents'
                )
            ),
            'actions' => array(
                Store::ACTION_LIST => array(
                    'type' => Store::ACTION_LIST,
                    'caption' => 'List'
                ),
                Store::ACTION_INSERT => array(
                    'type' => Store::ACTION_INSERT,
                    'caption' => 'Add'
                )
            )
        );
    }
}
