<?php

namespace tests\Bundle\Store\Field;

use Core;
use FestiTestCase;
use FieldException;
use Response;
use Store;

class TextareaFieldTest extends FestiTestCase
{
    const MAX_CHARS = 20;

    public function testMaxCharsLimit(): void
    {
        $options = array();

        $options['model'] = array(
            'table' => array(
                'name'       => 'site_contents',
                'primaryKey' => 'id',
                'exceptionMode' => true
            ),

            'fields' => array(
                'value' => array(
                    'type' => 'textarea',
                    'name' => 'name',
                    'caption' => __('Full Name'),
                    'maxChars' => static::MAX_CHARS
                )
            ),
            'actions' => array(
                Store::ACTION_LIST => array(
                    'type' => Store::ACTION_LIST,
                    'caption' => 'List'
                ),
                Store::ACTION_INSERT => array(
                    'type' => Store::ACTION_INSERT,
                    'caption' => 'Add'
                )
            )
        );

        $store = Core::getInstance()->createStoreInstance(
            'site_contents',
            $options
        );

        $action = $store->createActionInstance(Store::ACTION_INSERT);

        $values = array(
            'name' => 'Taras Petrov Ivanovich'
        );

        FestiTestCase::preparePostForStore($action, $values, $store);

        $response = new Response();

        $message = __(
            '%s cannot be longer than %s characters.',
            __('Full Name'),
            static::MAX_CHARS
        );

        try {
            $action->onStart($response);
            $this->fail('FieldException expected');
        } catch (FieldException $exception) {
            $this->assertEquals(
                $message,
                $exception->getMessage()
            );
        }
    }
}
