<?php

namespace tests\Bundle\Store;

use AbstractField;
use Core;
use DataAccessObject;
use FestiTestCase;
use ListAction;
use Response;
use Store;

class FieldFiltersTest extends FestiTestCase
{
    public function testAllFiltersType(): void
    {
        $values = array(
            'default' => array(
                array('name' => 'Test 1'),
                array('name' => 'Test 2')
            ),
            'select_filter' => array(
                array('id' => 10, 'name' => 'Test 1', 'id_parent' => null),
                array('id' => 11, 'name' => 'Test 2', 'id_parent' => null),
                array('id' => 12, 'name' => 'Test 3', 'id_parent' => 10),
                array('id' => 13, 'name' => 'Test 4', 'id_parent' => 10),
                array('id' => 14, 'name' => 'Test 5', 'id_parent' => 11),
                array('id' => 15, 'name' => 'Test 6', 'id_parent' => 11),
                array('id' => 16, 'name' => 'Test 7', 'id_parent' => 11),
            ),
            'checkbox' => array(
                array('name' => null),
                array('name' => 1)
            ),
            'checkbox_select' => array(
                array('name' => null),
                array('name' => 1)
            ),
            'datetime' => array(
                array('cdate' => '2018-01-01'),
                array('cdate' => '2018-05-01')
            ),
            'datetime_range' => array(
                array('cdate' => '2018-01-01'),
                array('cdate' => '2018-05-01')
            ),
            'range' => array(
                array('id_parent' => 100),
                array('id_parent' => 1000),
                array('id_parent' => 10000)
            ),
            'composite' => array(
                array('name' => 'Test 1', 'id_parent' => 100),
                array('name' => 'Test 2', 'id_parent' => 1000)
            ),
            'many2many' => array(
                'multiple' => true,
                'cnt' => 2,
                'site_contents' => array(
                    array('id' => 10, 'name' => 'Test 1'),
                    array('id' => 11, 'name' => 'Test 2')
                ),
                'settings' => array(
                    array('id' => 2, 'value' => 'Value 1', 'caption' => 'c1'),
                    array('id' => 3, 'value' => 'Value 2', 'caption' => 'c2')
                ),
                'site_contents2settings' => array(
                    array('id_site_content' => 10, 'id_setting' => 2),
                    array('id_site_content' => 10, 'id_setting' => 3),
                    array('id_site_content' => 11, 'id_setting' => 3)
                )
            ),
            'many2many_unique' => array(
                'multiple' => true,
                'cnt' => 2,
                'site_contents' => array(
                    array('id' => 10, 'name' => 'Test 1'),
                    array('id' => 11, 'name' => 'Test 2')
                ),
                'settings' => array(
                    array('id' => 2, 'value' => 'Value 1', 'caption' => 'c1'),
                    array('id' => 3, 'value' => 'Value 2', 'caption' => 'c2')
                ),
                'site_contents2settings' => array(
                    array('id_site_content' => 10, 'id_setting' => 2),
                    array('id_site_content' => 10, 'id_setting' => 3),
                    array('id_site_content' => 11, 'id_setting' => 3)
                )
            ),
            'many2many_text' => array(
                'multiple' => true,
                'cnt' => 2,
                'site_contents' => array(
                    array('id' => 10, 'name' => 'Test 1'),
                    array('id' => 11, 'name' => 'Test 2')
                ),
                'settings' => array(
                    array('id' => 2, 'value' => 'Value 1', 'caption' => 'c1'),
                    array('id' => 3, 'value' => 'Value 2', 'caption' => 'c2')
                ),
                'site_contents2settings' => array(
                    array('id_site_content' => 10, 'id_setting' => 2),
                    array('id_site_content' => 10, 'id_setting' => 3),
                    array('id_site_content' => 11, 'id_setting' => 3)
                )
            ),
            'foreignKey' => array(
                array('id' => 10, 'name' => 'Test 1', 'id_parent' => null),
                array('id' => 11, 'name' => 'Test 2', 'id_parent' => 10),
                array('id' => 12, 'name' => 'Test 3', 'id_parent' => 10),
                array('id' => 13, 'name' => 'Test 24', 'id_parent' => 12),
                array('id' => 14, 'name' => 'Test 25', 'id_parent' => 13),
                array('id' => 15, 'name' => 'Test 36', 'id_parent' => 14),
                array('id' => 16, 'name' => 'Test 7', 'id_parent' => 15),
            ),
        );
        
        $filtersTypes = array(
            'text' => array(
                'filter' =>  'text',
                'type' => 'text',
                'conditions' => array(
                    array('query' => 'Test 1', 'cnt' => 1)
                )
            ),
            'select' => array(
                'filter' =>  'select',
                'type'   => 'select',
                'options' => "1:Test 1\n2:Test 2",
                'conditions' => array(
                    array(
                        'query' => 'Test 1', 'cnt' => 1, 'filtersValuesCnt' => 2
                    )
                )
            ),
            'select_filter' => array(
                'filter' => 'select',
                'type' => 'select',
                'name' => 'id_parent',
                'conditions' => array(
                    array(
                        'query' => '10', 'cnt' => 2,
                    ),
                    array(
                        'query' => '11', 'cnt' => 3,
                    ),
                )
            ),
            'checkbox' => array(
                'filter' => 'checkbox',
                'type' => 'checkbox',
                'conditions' => array(
                    array(
                        'query' => 1, 'cnt' => 1
                    )
                )
            ),
            'checkbox_select' => array(
                'filter' => 'select',
                'type' => 'checkbox',
                'conditions' => array(
                    array(
                        'query' => 0, 'cnt' => 1
                    )
                )
            ),
            'datetime' => array(
                'filter' => 'datetime',
                'name' => 'cdate',
                'type' => 'datetime',
                'conditions' => array(
                    array(
                        'query' => '2018-01-01', 'cnt' => 1
                    )
                )
            ),
            'datetime_range' => array(
                'filter' => 'range',
                'type' => 'datetime',
                'name' => 'cdate',
                'conditions' => array(
                    array(
                        'query' => array('2018-01-01'), 'cnt' => 2 // >=
                    ),
                    array(
                        'query' => array(1 => '2018-02-01'), 'cnt' => 1 // <=
                    ),
                    array(
                        'query' => array('2018-02-01', '2018-06-01'), 'cnt' => 1
                    ) 
                )
            ),
            'range' => array(
                'filter' =>  'range',
                'type'   => 'price',
                'name'   => 'id_parent',
                'conditions' => array(
                    array(
                        'query' => array(10), 'cnt' => 3 // >=
                    ),
                    array(
                        'query' => array(1 => 2000), 'cnt' => 2 // <=
                    ),
                    array(
                        'query' => array(1000, 10000), 'cnt' => 2
                    )
                )
            ), 
            'multiple' => array(
                'filter' =>  'multiple',
                'type'   => 'text',
                'conditions' => array(
                    array(
                        'query' => array('Test 1'), 
                        'cnt' => 1, 
                        'filtersValuesCnt' => 2
                    ),
                    array(
                        'query' => array('Test 1', 'Test 2'), 
                        'cnt' => 2, 
                        'filtersValuesCnt' => 2
                    )
                )
            ),
            'text_select' => array(
                'filter' =>  'select',
                'type'   => 'text',
                'conditions' => array(
                    array(
                        'query' => 'Test 1', 'cnt' => 1, 'filtersValuesCnt' => 2
                    )
                )
            ),
            'exact' => array(
                'filter' => 'exact',
                'type' => 'text',
                'conditions' => array(
                    array('query' => 'Test 1', 'cnt' => 1),
                    array('query' => 'Test', 'cnt' => 0)
                )
            ),
            'composite' => array(
                'field' => array(
                    'name' => 'test',
                    'type' => 'composite',
                    'filter' => 'text',
                    'caption' => 'Test',
                    'options' => array(
                        array('value' => 'name'),
                        array('value' => 'id_parent')
                    )
                ),
                'conditions' => array(
                    array('query' => 'Test', 'cnt' => 2),
                    array('query' => '1000', 'cnt' => 1),
                )
            ),
            'many2many' => array(
                'field' => array(
                    'type' => 'many2many',
                    'filter' => 'select',
                    'caption' => 'Test',
                    'linkTable' => 'site_contents2settings',
                    'linkField' => 'id_site_content',
                    'linkForeignField' => 'id_setting',
                    'foreignTable' => 'settings',
                    'foreignKeyField' => 'id',
                    'foreignValueField' => 'caption'
                ),
                'conditions' => array(
                    array('query' => 3, 'cnt' => 2, 'filtersValuesCnt' => 2),
                    array('query' => 2, 'cnt' => 1, 'filtersValuesCnt' => 2),
                    array('query' => array(2, 3), 'cnt' => 2)
                )
            ),
            'many2many_unique' => array(
                'field' => array(
                    'type' => 'many2many',
                    'filter' => 'select',
                    'caption' => 'Test',
                    'linkTable' => 'site_contents2settings',
                    'linkField' => 'id_site_content',
                    'linkForeignField' => 'id_setting',
                    'foreignTable' => 'settings',
                    'foreignKeyField' => 'id',
                    'foreignValueField' => 'caption',
                    'unique' => true
                ),
                'conditions' => array(
                    array('query' => 'c2', 'cnt' => 2, 'filtersValuesCnt' => 2),
                    array('query' => 'c1', 'cnt' => 1, 'filtersValuesCnt' => 2),
                    array('query' => array('c1', 'c2'), 'cnt' => 2)
                )
            ),
            'many2many_text' => array(
                'field' => array(
                    'type' => 'many2many',
                    'filter' => 'text',
                    'caption' => 'Test',
                    'linkTable' => 'site_contents2settings',
                    'linkField' => 'id_site_content',
                    'linkForeignField' => 'id_setting',
                    'foreignTable' => 'settings',
                    'foreignKeyField' => 'id',
                    'foreignValueField' => 'caption',
                    'unique' => true
                ),
                'conditions' => array(
                    array('query' => 'c2', 'cnt' => 2, 'filtersValuesCnt' => 2),
                    array('query' => 'c1', 'cnt' => 1, 'filtersValuesCnt' => 2),
                    array('query' => 'c',  'cnt' => 2),
                    array('query' => 'c3', 'cnt' => 0)
                )
            ),
            'foreignKey' => array(
                'field' => array(
                    'name'              => 'id_parent',
                    'type'              => 'foreignKey',
                    'filter'            => 'text',
                    'caption'           => 'Test',
                    'foreignTable'      => 'site_contents',
                    'foreignKeyField'   => 'id',
                    'foreignValueField' => 'name',
                    'alias'             => 'test_parent'
                ),
                'conditions' => array(
                    array('query' => '2', 'cnt' => 2),
                    array('query' => '3', 'cnt' => 2),
                    array('query' => 'Test', 'cnt' => 6)
                )
            ),
        );

        foreach ($filtersTypes as $testName => $filterTypeData) {
            $testValues = $values['default'];
            if (array_key_exists($testName, $values)) {
                $testValues = $values[$testName];
            }
            
            $this->_testFilter($filterTypeData, $testValues, $testName);
        }
    }

    private function _prepareData($testValues): int
    {
        if (!empty($testValues['multiple'])) {
            $expectedCnt = $testValues['cnt'];
            unset($testValues['multiple'], $testValues['cnt']);

            foreach ($testValues as $tableName => $values) {
                $this->_updateTestData($tableName, $values);
            }
            
            return $expectedCnt;
        }

        $storeName = 'site_contents';
        $this->_updateTestData($storeName, $testValues);

        return count($testValues);
    }

    private function _updateTestData($storeName, $testValues): void
    {
        $connection = $this->getDbConnection();

        $hasPrimaryKey = array_key_exists('id', $testValues[0]);

        if ($hasPrimaryKey && $connection->getDatabaseType() === DataAccessObject::TYPE_MSSQL) {
            $connection->query("SET IDENTITY_INSERT ".$storeName." ON");
        }

        $connection->query("TRUNCATE TABLE ".$storeName);

        $connection->massInsert($storeName, $testValues);

        if ($hasPrimaryKey && $connection->getDatabaseType() === DataAccessObject::TYPE_MSSQL) {
            $connection->query("SET IDENTITY_INSERT ".$storeName." OFF");
        }
    }

    private function _testFilter($filterTypeData, $testValues, $testName): void
    {
        $storeName = 'site_contents';
        
        // Remove all records
        $expectedCnt = $this->_prepareData($testValues);
        
        $dgsModel = $this->_getDefaultModel();
        
        if (!empty($filterTypeData['field'])) {
            $field = $filterTypeData['field'];
        } else {
            $field = $this->_getDefaultFieldModel($filterTypeData);
        }
        
        if (empty($field['name'])) {
            $filedName = 0;
        } else {
            $filedName = $field['name'];
        }
        
        $dgsModel['model']['fields'] = array(
            $filedName => $field
        );

        $store = Core::getInstance()->createStoreInstance(
            $storeName,
            $dgsModel
        );
        
        // Check default amount of rows.

        $listAction = $store->createActionInstance(Store::ACTION_LIST);
        $response = new Response();
        $listAction->onStart($response);
        $values = $listAction->getValues();

        $currentCnt = count($values);
        $message = "Filter '".$testName."': expected ".$expectedCnt." but get ".$currentCnt." ".print_r($values, 1);
        $this->assertTrue($currentCnt == $expectedCnt, $message);

        $filed = $store->getModel()->getFieldByName($filedName);

        foreach ($filterTypeData['conditions'] as $condition) {
            $filters = array(
                $filedName => $condition['query']
            );

            FestiTestCase::prepareFiltersForStore($filters, $store);
            $listAction->onStart($response);
            $values = $listAction->getValues();

            $message = "Filter '".$testName."' [".$filedName."]: expected ".$condition['cnt'].
                        " but get ".count($values)." ".print_r($values, 1);
            $this->assertTrue(count($values) == $condition['cnt'], $message);
            
            if (!empty($condition['filtersValuesCnt'])) {
                $cnt = $condition['filtersValuesCnt'];
                $filterValues = $filed->getFilterValues();
                $this->assertTrue(
                    count($filterValues) == $cnt, 
                    'filters values has error'
                );
            }
            
            FestiTestCase::cleanFiltersForStore($store);
        }
    } // end _testFilter

    private function _getDefaultFieldModel(array $filterTypeData): array
    {
        $filedName = "name";
        if (!empty($filterTypeData['name'])) {
            $filedName = $filterTypeData['name'];
        }
        
        $options = null;
        if (!empty($filterTypeData['options'])) {
            $options = $filterTypeData['options'];
        }
        
        return array(
            'type'    => $filterTypeData['type'],
            'name'    => $filedName,
            'caption' => 'Value',
            'filter'  => $filterTypeData['filter'],
            'options' => $options
        );
    } // end _getDefaultFieldModel

    private function _getDefaultModel(): array
    {
        $options = array();

        $options['model'] = array(
            'table' => array(
                'name'          => 'site_contents',
                'primaryKey'    => 'id',
                'exceptionMode' => true
            ),

            'fields' => array(),

            'actions' => array(
                Store::ACTION_LIST => array(
                    'type' => Store::ACTION_LIST,
                    'caption' => 'Edit'
                ),
                Store::ACTION_INSERT => array(
                    'type' => Store::ACTION_INSERT,
                    'caption' => 'Edit'
                )
            )
        );
        
        return $options;
    } // end _getDefaultModel

    public function testFilterByAggregationColumn(): void
    {
        $core = $this->getCoreWithConnection();

        $values = array(
            array(
                'caption' => 'Test',
                'value' => 2,
                'amount' => 3
            ),
            array(
                'caption' => 'Test',
                'value' => 2,
                'amount' => 2
            )
        );

        $expectedSumValue = 0;
        foreach ($values as $row) {
            $expectedSumValue += $row['amount'];
        }

        $core->db->massInsert(STORE_DEFAULT, $values);

        $store = $core->createStoreInstance('filter_by_aggregation_column');

        $store->setOption("storeExceptionMode", true);

        $action = $store->createActionInstance(Store::ACTION_LIST);
        assert($action instanceof ListAction);

        $this->assertInstanceOf(ListAction::class, $action);

        // Aggregate Function column
        $response = new Response();
        $action->onStart($response);
        $this->assertNotEmpty($response->content);
        $values = $action->getValues();
        $this->assertTrue(count($values) == 1);
        list($row) = $values;
        $this->assertTrue($row['sum_column'] == $expectedSumValue);

        // composite positive
        $filters = array(
            'value' => 'Test'
        );

        FestiTestCase::prepareFiltersForStore($filters, $store);
        $response = new Response();
        $action->onStart($response);
        $values = $action->getValues();
        list($row) = $values;
        $this->assertTrue($row['sum_column'] == $expectedSumValue);
        FestiTestCase::cleanFiltersForStore($store);

        // composite negative
        $filters = array(
            'value' => 'AAAAAAA'
        );

        FestiTestCase::prepareFiltersForStore($filters, $store);
        $response = new Response();
        $action->onStart($response);
        $values = $action->getValues();
        $this->assertTrue(empty($values));
        FestiTestCase::cleanFiltersForStore($store);

        // having
        $this->_testHavingByField($action, $store, $expectedSumValue);
    }

    private function _testHavingByField(ListAction &$action, Store &$store, $expectedSumValue): void
    {
        $filters = array(
            'sum_column' => $expectedSumValue
        );

        $values = $this->_acceptFilter($filters, $store, $action);
        $this->assertTrue(count($values) == 1);

        list($row) = $values;
        $this->assertTrue($row['sum_column'] == $expectedSumValue);

        //
        $sumField = $store->getModel()->getFieldByName('sum_column');

        $sumField->set(AbstractField::OPTION_FILTER, AbstractField::FILTER_TYPE_RANGE);

        $filters = array(
            'sum_column' => array(1, $expectedSumValue)
        );

        $values = $this->_acceptFilter($filters, $store, $action);

        $this->assertTrue(count($values) == 1);

        list($row) = $values;
        $this->assertTrue($row['sum_column'] == $expectedSumValue);
    } // end _testHavingByField

    private function _acceptFilter($filters, Store $store, ListAction $action): array
    {
        FestiTestCase::prepareFiltersForStore($filters, $store);
        $response = new Response();
        $action->onStart($response);
        $values = $action->getValues();
        FestiTestCase::cleanFiltersForStore($store);

        return $values;
    }

    public function testInsensitiveSearch(): void
    {
        $core = $this->getCoreWithConnection();

        $expectedValuesCount = 3;
        $values = array(
            array(
                'name' => 'Test',
            ),
            array(
                'name' => 'other value',
            ),
            array(
                'name' => 'TEST',
            ),
            array(
                'name' => 'test',
            )
        );

        $this->_updateTestData('site_contents', $values);

        $store = $core->createStoreInstance('site_contents');

        $field = $store->getModel()->getFieldByName('name');
        $field->set(AbstractField::OPTION_FILTER, 'text');

        FestiTestCase::cleanFiltersForStore($store);

        $filters = array(
            'name' => 'test'
        );

        FestiTestCase::prepareFiltersForStore($filters, $store);

        $action = $store->createActionInstance(Store::ACTION_LIST);
        assert($action instanceof ListAction);

        $response = new Response();
        $action->onStart($response);
        $values = $action->getValues();

        $this->assertEquals(count($values), $expectedValuesCount);
    }
}
