<?php

namespace tests\Bundle\Store;

use DataAccessObject;
use FestiTestCase;
use ListAction;
use Response;
use Store;
use StoreException;

class FiltersTest extends FestiTestCase
{
    public function testFilters(): void
    {
        $core = $this->getCoreWithConnection();

        $expectedID = 1;
        $values = array(
            array(
                'id' => $expectedID,
                'caption' => 'Test',
                'value' => 2,
                'amount' => 3
            ),
            array(
                'id' => $expectedID + 1,
                'caption' => 'Test',
                'value' => 2,
                'amount' => 2
            )
        );

        if ($core->db->getDatabaseType() === DataAccessObject::TYPE_MSSQL) {
            $core->db->query("SET IDENTITY_INSERT ".STORE_DEFAULT." ON");
        }

        $core->db->massInsert(STORE_DEFAULT, $values);

        if ($core->db->getDatabaseType() === DataAccessObject::TYPE_MSSQL) {
            $core->db->query("SET IDENTITY_INSERT ".STORE_DEFAULT." OFF");
        }

        $store = $core->createStoreInstance('dgs_filters');

        $action = $store->createActionInstance(Store::ACTION_LIST);
        assert($action instanceof ListAction);

        // Use filter
        $response = new Response();
        $action->onStart($response);
        $values = $action->getValues();

        $this->assertTrue(count($values) == 1);
        list($row) = $values;
        $this->assertTrue($row['id'] == $expectedID);

        // Undefined filter value
        $filters = &$store->getModel()->getFilters();
        $filters['id'] = null;
        $hasException = false;
        try {
            $action->onStart($response);
        } catch (StoreException $exp) {
            $hasException = true;
        }

        $this->assertTrue($hasException);

        // Session value
        $session = &$store->getSession();
        $session['expected_id'] = $expectedID;
        $filters['id'] = 'S%expected_id%';
        $action->onStart($response);
        $values = $action->getValues();
        list($row) = $values;
        $this->assertTrue($row['id'] == $expectedID);

        // Session value null
        $session = &$store->getSession();
        unset($session['expected_id']);
        $filters['id'] = 'S%expected_id%';
        $action->onStart($response);
        $values = $action->getValues();
        $this->assertEmpty($values);

        // NULL value
        $filters['id'] = 'NULL';
        $action->onStart($response);
        $values = $action->getValues();
        $this->assertEmpty($values);
    }

    public function testParentFilter(): void
    {
        $core = $this->getCoreWithConnection();

        $parentValue = 1;
        $expectedID = 2;
        $values = array(
            array(
                'id' => $parentValue,
                'caption' => 'Test',
                'id_parent' => null
            ),
            array(
                'id' => $expectedID,
                'caption' => 'Test',
                'id_parent' => 1
            )
        );

        if ($core->db->getDatabaseType() === DataAccessObject::TYPE_MSSQL) {
            $core->db->query("SET IDENTITY_INSERT ".STORE_DEFAULT." ON");
        }

        $core->db->massInsert(STORE_DEFAULT, $values);

        if ($core->db->getDatabaseType() === DataAccessObject::TYPE_MSSQL) {
            $core->db->query("SET IDENTITY_INSERT ".STORE_DEFAULT." OFF");
        }

        $store = $core->createStoreInstance('dgs_filters');

        $action = $store->createActionInstance(Store::ACTION_LIST);
        assert($action instanceof ListAction);

        $filters = &$store->getModel()->getFilters();
        $filters['id'] = $expectedID;

        $store->setSessionValue(Store::PARENT_PRIMARY_KEY_IN_SESSION, $parentValue);

        $response = new Response();
        $action->onStart($response);
        $values = $action->getValues();

        list($row) = $values;
        $this->assertTrue($row['id'] == $expectedID);
    }
}
