<?php

namespace tests\Bundle\Store;

use EditAction;
use FestiEvent;
use FestiTestCase;
use InsertAction;
use Response;
use Store;

class MultipleStoreTest extends FestiTestCase
{
    public function testDuplicateValuesAfterEdit(): void
    {
        $core = $this->getCoreWithConnection();

        $pluginName = 'MultipleStoreTest';
        $values = array(
            'status' => 'active',
            'ident'  => $pluginName
        );

        $core->db->insert('festi_plugins', $values);

        //
        $store = $core->createStoreInstance('dgs_multiple_tables');
        $store->setOption("storeExceptionMode", true);

        $action = $store->createActionInstance(Store::ACTION_INSERT);

        $this->assertInstanceOf(InsertAction::class, $action);

        $values = array(
            'area'    => 'backend',
            'plugin'  => $pluginName,
            'pattern' => 'asdasd',
            'method'  => 'sdsdf'
        );

        FestiTestCase::preparePostForStore(
            Store::ACTION_INSERT,
            $values,
            $store
        );

        $response = new Response();

        $action->onStart($response);

        $primaryKeyValue = $action->getPrimaryKeyValue();

        //
        $action = $store->createActionInstance(Store::ACTION_EDIT);
        assert($action instanceof EditAction);
        $action->load($primaryKeyValue);

        $values['method'] = 'sdsdf2';

        FestiTestCase::preparePostForStore(
            Store::ACTION_EDIT,
            $values,
            $store
        );

        $action->onStart($response);

        //
        $search = array(
            'plugin' =>  $pluginName
        );

        $sql = "SELECT * FROM festi_url_rules";

        $rows = $core->db->select($sql, $search);

        $this->assertTrue(count($rows) == 1, 'Duplicate rows has been found into the additional store');

        $core->db->delete('festi_url_rules', array());
        $core->db->delete('festi_url_rules2areas', array());

        $this->_testAddExternalValuesToChildTables($store);

        $core->db->delete('festi_url_rules', array());
        $core->db->delete('festi_url_rules2areas', array());
        $core->db->delete('festi_plugins', array());
    }

    public function _testAddExternalValuesToChildTables(Store $store): void
    {
        $expectedValues = 'EXTERNAL VALUES';

        $store->addEventListener(
            Store::EVENT_PREPARE_REPOSITORY_VALUES,
            static function (FestiEvent &$event) use ($expectedValues) {
                $tablesValues = &$event->getTargetValueByKey('general');
                $store = &$event->getTargetValueByKey('store');
                assert($store instanceof Store);

                $tablesValues['festi_url_rules']['method'] = $expectedValues;
            }
        );

        $action = $store->createActionInstance(Store::ACTION_INSERT);

        $this->assertInstanceOf(InsertAction::class, $action);

        $values = array(
            'area' => 'backend',
            'plugin' => 'MultipleStoreTest',
            'pattern' => 'asdasd',
            'method' => 'sdsdf'
        );

        FestiTestCase::preparePostForStore(
            Store::ACTION_INSERT,
            $values,
            $store
        );

        $response = new Response();
        $action->onStart($response);

        $sql = "SELECT * FROM festi_url_rules";

        $core = $this->getCoreWithConnection();
        $row = $core->db->getRow($sql);

        $this->assertTrue($row['method'] ==  $expectedValues);
    }

    public function testUpdateOnlyForeignTableValuesAndComplexRouteJoin(): void
    {
        $store = $this->core->createStoreInstance('dgs_multiple_tables_complex_route_join');

        $pluginName = 'MultipleStoreTest';
        $values = array(
            'status' => 'active',
            'ident'  => $pluginName
        );

        $this->core->db->insert("festi_plugins", $values);

        $action = $store->createActionInstance(Store::ACTION_INSERT);

        $this->assertInstanceOf(InsertAction::class, $action);

        $values = array(
            'area' => 'backend',
            'pattern' => 'asdasd',
            'method' => 'sdsdf'
        );

        FestiTestCase::preparePostForStore(
            Store::ACTION_INSERT,
            $values,
            $store
        );

        $response = new Response();

        $action->onStart($response);

        $primaryKeyValue = $action->getPrimaryKeyValue();

        $sql = "SELECT * FROM festi_url_rules WHERE id = ".$primaryKeyValue;

        $row = $this->core->db->getRow($sql);

        $this->assertTrue($row['plugin'] == $pluginName);

        //
        $action = $store->createActionInstance(Store::ACTION_EDIT);
        assert($action instanceof EditAction);
        $action->load($primaryKeyValue);

        $values['method'] = 'updated_method';

        FestiTestCase::preparePostForStore(
            Store::ACTION_EDIT,
            $values,
            $store
        );

        $action->onStart($response);

        $row = $this->core->db->getRow($sql);

        $this->assertTrue($row['method'] == $values['method']);
        $this->assertTrue($row['plugin'] == $pluginName);
    }
}
