<?php

namespace tests\Bundle\Store;

use FestiTestCase;
use InsertAction;
use Response;
use Store;
use StoreModel;

class SectionsTest extends FestiTestCase
{
    public function testFormTabsDefaultMode(): void
    {
        $extendOptions = array(
            'fields' => array(
                'field_1' => array(
                    'section' => 'tab_1'
                ),
                'field_2' => array(
                    'section' => 'tab_1'
                ),
                'field_3' => array(
                    'section' => 'tab_2'
                ),
            )
        );

        $store = $this->_getBaseStore($extendOptions, 3);

        $this->_runAsserts($store, $extendOptions);
    } // end testFormTabsDefaultMode

    public function testFormTabsDefaultModeWithoutTab(): void
    {
        $extendOptions = array(
            'fields' => array(
                'field_1' => array(
                    'section' => 'tab_1'
                ),
                'field_2' => array(
                    'section' => 'tab_2'
                ),
                'field_3' => array(
                    'section' => 'tab_3' // invalid section
                ),
                // field_4 - without section
            )
        );

        $store = $this->_getBaseStore($extendOptions, 4);

        $extendOptions['fields']['field_3']['section'] = 'tab_1';
        $extendOptions['fields']['field_4']['section'] = 'tab_1';

        $this->_runAsserts($store, $extendOptions);
    } // end testFormTabsDefaultModeWithoutTab

    public function testFormTabsDefaultModeWithoutFields(): void
    {
        $extendOptions = array(
            'fields' => array(
                'field_1' => array(
                    'section' => 'tab_1'
                ),
                // field_2, field_3 - without section
            )
        );

        $store = $this->_getBaseStore($extendOptions, 3);

        $extendOptions['fields']['field_2']['section'] = 'tab_1';
        $extendOptions['fields']['field_3']['section'] = 'tab_1';

        $this->_runAsserts($store, $extendOptions);

    } // end testFormTabsDefaultModeWithoutFields

    public function testFormTabsNewModeWithoutTab(): void
    {
        $extendOptions = array(
            'fields' => array(
                'field_1' => array(
                    'section' => 'tab_1'
                ),
                'field_2' => array(
                    'section' => 'tab_2'
                ),
                'field_3' => array(
                    'section' => 'tab_3' // invalid section
                ),
                // field_4 - without section
            )
        );

        $store = $this->_getBaseStore($extendOptions, 4);

        $extendOptions['fields']['field_3']['section'] = 'tab_1';
        $extendOptions['fields']['field_4']['section'] = 'tab_1';

        $this->_runAsserts($store, $extendOptions);
    } // end testFormTabsDefaultModeWithoutTab

    public function testFormSectionsDefaultMode(): void
    {
        $extendOptions = array(
            'fields' => array(
                'field_1' => array(
                    'section' => 'tab_1'
                ),
                'field_2' => array(
                    'section' => 'tab_1'
                ),
                'field_3' => array(
                    'section' => 'tab_2'
                ),
            ),
            'sections' => array(
                '@attributes' => array(
                    'mode' => StoreModel::OPTION_SECTIONS_MODE_WIDGETS
                )
            ),
        );

        $store = $this->_getBaseStore($extendOptions, 3);

        $extendOptions['sections'] = array();

        $this->_runAsserts($store, $extendOptions);
    } // end testFormSectionsDefaultMode

    public function testFormSectionsNewMode(): void
    {
        $extendOptions = array(
            'fields' => array(
                'field_1' => array(
                    'section' => 'tab_1'
                ),
                'field_2' => array(
                    'section' => 'tab_1'
                ),
                'field_3' => array(
                    'section' => 'tab_2'
                ),
            ),
            'sections' => array(
                '@attributes' => array(
                    'mode' => StoreModel::OPTION_SECTIONS_MODE_WIDGETS
                )
            ),

            'actions' => array(
                Store::ACTION_INSERT => array(
                    'mode'    => 'new'
                )
            )
        );

        $store = $this->_getBaseStore($extendOptions, 3);

        $this->_runAsserts($store, $extendOptions, false);
    } // end testFormSectionsNewMode

    private function _runAsserts($store, $extendOptions, $isTabs = true): void
    {
        $formContent = $this->_getFormContent($store);

        $sections = $this->_getFormSections($formContent, $isTabs);

        $this->assertEquals(
            count($extendOptions['sections']),
            count($sections)
        );

        foreach ($extendOptions['sections'] as $tabName => $tabInfo) {
            $tabFields = $this->_getFields($formContent, $tabName, $isTabs);

            $expectedTabFields = $this->_getExpectedTabFields(
                $extendOptions['fields'],
                $tabName
            );

            $this->assertEquals(
                count($tabFields),
                count($expectedTabFields)
            );

            $this->assertEmpty(
                array_diff($tabFields, $expectedTabFields)
            );

            $this->assertEmpty(
                array_diff($expectedTabFields, $tabFields)
            );
        }

    } // end _runAsserts

    private function _getFormSections(string $formContent, bool $isTab = true): array
    {
        $sections = array();

        if ($isTab) {
            $regExp = "#<a data-toggle=\"tab\" href=\"\#(tab_\d+)\">\s*".
                "<div class=\"tab-item-title\">".
                "(Tab \d+)<\/div>\s*<\/a>#Umis";
        } else {
            $regExp = "#_start_section_(\w+)_\s+<header>([\w\s]+)".
                "<\/header>#Umis";
        }

        $result = preg_match_all(
            $regExp,
            $formContent,
            $matches,
            PREG_SET_ORDER
        );

        if ($result == false || !count($matches)) {
            return $sections;
        }

        foreach ($matches as $match) {
            $sections[$match[1]] = $match[2];
        }

        return $sections;
    } // end _getFormSections

    private function _getSectionContent(string $formContent, string $section, bool $isTab = true): string
    {
        if ($isTab) {
            $regExp = "#<div id=\"".$section.
                "\" class=\"tab-pane[^\"]+\">(.*)<\/div>\s*<\/div>#Umis";
        } else {
            $regExp = "#_start_section_".$section.
                "_(.*)_end_section_".$section."_#Umis";
        }
        $result = preg_match_all(
            $regExp,
            $formContent,
            $matches,
            PREG_SET_ORDER
        );

        if ($result == false || !count($matches)) {
            return false;
        }

        return trim($matches[0][1]);
    } // end _getSectionContent

    private function _getFields(string $formContent, string $section, bool $isTab = true): array
    {
        $fields = array();

        $content = $this->_getSectionContent($formContent, $section, $isTab);

        $result = preg_match_all(
            "#<input[^>]+name=\"(\w+)\"[^>]+\/>#Umis",
            $content,
            $matches,
            PREG_SET_ORDER
        );

        if ($result == false || !count($matches)) {
            return $fields;
        }

        foreach ($matches as $match) {
            $fields[] = $match[1];
        }

        return $fields;
    } // end _getFields

    private function _getFormContent(Store $store): string
    {
        $action = $store->createActionInstance(Store::ACTION_INSERT);

        $this->assertInstanceOf(InsertAction::class, $action);

        $response = new Response();

        $action->onStart($response);

        $content = $response->getContent();
        $this->assertNotEmpty($content);

        $regExps = array(
            "#<div class=\"input-group\">(.+)<\/div>#Umis",
            "#<div class=\"col-sm-\d+ e-input\">(.+)<\/div>#Umis",
            "#<div class=\"form-group e-item e-item-(?:even|odd)\">".
                "(.+)<\/div>#Umis",
            "#<div class=\"col-xs-\d+ col-md-\d+\">(.+)<\/div>#Umis"
        );

        $content = preg_replace(
            $regExps,
            "$1",
            $content
        );

        $content = preg_replace(
            "#<div class=\"[^>]+f-db-form-section section-(\w+)\">".
            "(.+)<\/div>#Umis",
            "_start_section_$1_\n$2\n_end_section_$1_",
            $content
        );

        $content = preg_replace(
            array(
                "#<div class=\"row\">(.+)<\/div>#Umis",
                "#<div class=\"clearfix\"><\/div>#Umis"
            ),
            array("$1", ""),
            $content
        );

        return $content;
    } // end _getFormContent

    private function _getDefaultStoreModel(int $countFields): array
    {
        $options = array();

        $fields = array();

        foreach (range(1, $countFields) as $i) {
            $fieldName = 'field_'.$i;
            $fields[$fieldName] = array(
                'type'      => 'text',
                'name'      => $fieldName,
                'caption'   => ucfirst($fieldName),
                'required'  => 'true',
            );
        }

        $options['model'] = array(
            'table'   => array(
                'name'          => 'tmp_store',
                'primaryKey'    => 'id',
                'exceptionMode' => true,
            ),

            'sections' => array(
                '@attributes' => array(
                    'mode' => StoreModel::OPTION_SECTIONS_MODE_TABS,
                ),
                'tab_1' => array(
                    'name' => 'tab_1',
                    'caption' => 'Tab 1'
                ),
                'tab_2' => array(
                    'name' => 'tab_2',
                    'caption' => 'Tab 2'
                )
            ),

            'fields'  => $fields,

            'actions' => array(
                Store::ACTION_INSERT => array(
                    'type'    => Store::ACTION_INSERT,
                    'caption' => 'Add'
                )
            )
        );

        return $options;
    } // end _getDefaultStoreModel

    private function _getBaseStore(?array &$extendOptions = null, int $countFields = 2): Store
    {
        $core = $this->getCoreWithConnection();

        $options = $this->_getDefaultStoreModel($countFields);

        if (!empty($extendOptions)) {
            $options['model'] = array_merge_recursive(
                $options['model'],
                $extendOptions
            );

            $extendOptions = $options['model'];
        }

        $store = $core->createStoreInstance('tmp_store', $options);

        unset($extendOptions['sections']['@attributes']);

        return $store;
    } // end _getBaseStore

    private function _getExpectedTabFields(array $fields, string $tab): array
    {
        $fields = array_filter(
            $fields,
            static function ($fieldInfo) use ($tab) {
                return $tab == $fieldInfo['section'];
            }
        );

        return array_keys($fields);
    } // end _getExpectedTabFields
}
