<?php

namespace tests\Bundle\Store;

use Core;
use DataAccessObject;
use DatabaseException;
use DefaultUser;
use FestiTestCase;

class StoreAuditTest extends FestiTestCase
{
    private const TEST_TABLE_NAME       = 'site_contents';
    private const TEST_AUDIT_TABLE_NAME = '__site_contents';

    public function testAuditTable(): void
    {
        $sql = "SELECT 1 FROM __site_contents LIMIT 1";

        try {
            $this->core->db->getRow($sql);
            $this->fail("Table ".static::TEST_AUDIT_TABLE_NAME." exists");
        } catch (DatabaseException $exp) {
            $this->assertTrue($exp->getQuery() == $sql);
        }

        // TODO:
        if ($this->core->db->getDatabaseType() !== DataAccessObject::TYPE_MSSQL) {
            $this->_checkIndexes(static::TEST_TABLE_NAME, 1);
            $this->_addIndex();
            $this->_checkIndexes(static::TEST_TABLE_NAME, 2);
        }

        $core = &Core::getInstance();
        $params = array(
            'auth'       => 'yes',
            'auth_id'    => '1',
            'auth_login' => 'admin',
            'auth_role'  => '1'
        );
    
        $core->user = new DefaultUser($params);
    
        $this->doTestStore('site_contents_audit_mode');
    
        $sql = 'SELECT * FROM '.static::TEST_AUDIT_TABLE_NAME;
        $result = $this->core->db->getAll($sql);
        
        $countActions = 3;
        $this->assertCount($countActions, $result);

        // TODO:
        if ($this->core->db->getDatabaseType() != DataAccessObject::TYPE_MSSQL) {
            $this->_checkIndexes(static::TEST_AUDIT_TABLE_NAME, 1);
        }
    }

    private function _addIndex(): void
    {
        $sql = 'ALTER TABLE '.$this->core->db->quoteTableName(static::TEST_TABLE_NAME).
                ' ADD UNIQUE ('.$this->core->db->quoteColumnName('cdate').');';
        $this->core->db->query($sql);
    }

    private function _checkIndexes($table, $expectedCount): void
    {
        $indexes = $this->core->db->getTableIndexes($table);
        $this->assertCount($expectedCount, $indexes);
    }
}
