<?php

class ThemeTest extends FestiTestCase
{
    /**
     * @var Core
     */
    protected $core;

    /**
     * @var ISystemPlugin
     */
    protected $systemPlugin;

    protected function setUp(): void
    {
        parent::setUp();

        $this->core = Core::getInstance();
    }

    public function testChangeLogoImageAndLink(): void
    {
        $this->core->addEventListener(
            ITheme::EVENT_THEME_ON_PREPARE_LOGO,
            function (FestiEvent &$event) {
                $target = &$event->getTarget();

                $target['link'] = 'example.com';
                $target['logo'] = '/test-logo.png';
            }
        );

        $result = $this->_getDisplayResult();

        $pattern = '/id="logo".*href="example\.com".*src="\/test-logo\.png"/s';
        $this->assertMatchesRegularExpression($pattern, $result);
    }

    public function testChangeLogoContent(): void
    {
        $this->core->addEventListener(
            ITheme::EVENT_THEME_ON_PREPARE_LOGO,
            function (FestiEvent &$event) {
                $target = &$event->getTarget();

                $target['content'] = 'test logo content';
            }
        );

        $result = $this->_getDisplayResult();

        $pattern = '/id="logo".*test logo content/s';
        $this->assertMatchesRegularExpression($pattern, $result);
    }

    public function testAddContentRight(): void
    {
        $testContent = 'testAddContentRight';

        $this->core->addEventListener(
            ITheme::EVENT_THEME_FETCH_HEADER_ELEMENT,
            function (ThemeEvent &$event) use ($testContent) {

                $position = $event->getTargetValueByKey('position');

                if ($position == "right") {
                    return $testContent;
                }

                return null;
            }
        );

        $result = $this->_getDisplayResult();

        $this->assertTrue(is_int(strpos($result, $testContent)));
    }

    private function _getDisplayResult(): string
    {
        $_SERVER['REQUEST_URI'] = '/';

        $content = '';
        $displayPlugin = new DisplayPlugin();

        return $displayPlugin->fetchMain($content);
    }

}
