<?php

class FestiTestCase extends PHPUnit\Framework\TestCase
{
    public const EVENT_ON_HAS_USER_PERMISSION_TO_SECTION = 'has_user_permission_to_section';
    static private $pdo = null;

    /**
     * @var Core
     */
    protected $core;

    protected function setUp(): void
    {
        $this->doDump("create");

        $options = array(
            'path'                    => FS_PROJECT_ROOT,
            'engine_folder'           => '',
            'defs_path'               => FS_RESOURCES_ROOT.'defs'.DIRECTORY_SEPARATOR,
            'theme_path'              => FS_THEME_ROOT,
            'theme_template_path'     => FS_THEME_ROOT.'templates'.DIRECTORY_SEPARATOR,
            Core::OPTION_PLUGINS_PATH => FS_RESOURCES_ROOT.'plugins'.DIRECTORY_SEPARATOR
        );

        $this->core = Core::getInstance($options);

        $_REQUEST = array();
        $_GET     = array();
        $_POST    = array();
    } // end setUp
    
    protected function tearDown(): void
    {
        $this->doDump("clean");
    }

    public function &getCoreWithConnection()
    {
        $core = Core::getInstance();

        $this->assertNotNull($core);

        $core->db = $this->getDbConnection();

        return $core;
    } // end getCoreWithConnection
    
    public function &getControllerWithConnection()
    {
        return $this->getCoreWithConnection();
    } // end getControllerWithConnection
    
    public function getDataSet()
    {
        return $this->createXMLDataSet('Resources/default_dataset.xml');
    } // end getDataSet
    
    public function getDbConnection()
    {
        if (self::$pdo == null) {
            self::$pdo = new PDO(
                $GLOBALS['config']['db']['dsn'],
                $GLOBALS['config']['db']['user'],
                $GLOBALS['config']['db']['pass']
            );
        }

        return DataAccessObject::factory(self::$pdo);
    } // end getDbConnection
    
    protected function getDefaultStore($params = array())
    {
        return $this->core->createStoreInstance(STORE_DEFAULT, $params);
    } // end getDefaultStore
    
    public static function preparePostForStore(
        $action, $values, &$store, $primaryKeyValue = false
    )
    {
        $storeName = $store->getIdent();
        $_POST[$storeName] = array();
        $_REQUEST[$storeName] = array();
        
        $key = Store::ACTION_PERFORM_KEY_IN_POST;
        $_POST[$storeName][$key] = Store::ACTION_PERFORM_SAVE;
        
        foreach ($values as $key => $value) {
            $_POST[$key] = $value;
            $_REQUEST[$key] = $value;
        }
        
        $_GET[$storeName][Store::ACTION_KEY_IN_REQUEST] = $action;
        $_REQUEST[$storeName][Store::ACTION_KEY_IN_REQUEST] = $action;
        
        if ($primaryKeyValue) {
            $key = Store::PRIMARY_KEY_IN_REQUEST;
            $_GET[$storeName][$key] = $primaryKeyValue;
            $_REQUEST[$storeName][$key] = $primaryKeyValue;
        }
        
        $store->setRequest($_REQUEST[$storeName]);
    } // end preparePostForStore
    
    public static function prepareFiltersForStore($values, &$store)
    {
        $storeName = $store->getIdent();
        
        $_GET[$storeName]['filter_wtd'] = 'apply';
        
        foreach ($values as $key => $value) {
            $_GET[$storeName]['filter'][$storeName.'_'.$key] = $value;
        }
        
        return true;
    } // end prepareFiltersForStore
    
    public static function cleanFiltersForStore(&$store)
    {
        unset($_GET[$store->getIdent()]);
        
        $session = &$store->getSession();
        $session[Store::FILTERS_KEY_IN_SESSION] = array();
        
        return true;
    } // end cleanFiltersForStore

    public static function cleanPostByStoreName($storeName)
    {
        $_POST[$storeName] = array();
        $_REQUEST[$storeName] = array();
        $_GET[$storeName] = array();
    } // end cleanPostByStoreName

    /**
     * @param Store $store
     * @return void
     */
    public static function cleanPostByStore(Store &$store): void
    {
        static::cleanPostByStoreName($store->getName());

        $store->setRequestParam(Store::ACTION_KEY_IN_REQUEST, null);
        $store->setRequestParam(Store::PRIMARY_KEY_IN_REQUEST, null);
    }

    public static function preparePostByStoreName(
        $storeName, 
        $values, 
        $primaryKeyValue = false
    )
    {
        $action = $primaryKeyValue ? Store::ACTION_EDIT : Store::ACTION_INSERT;
        
        $_POST[$storeName] = array();
        $_REQUEST[$storeName] = array();
        
        $key = Store::ACTION_PERFORM_KEY_IN_POST;
        $_POST[$storeName][$key] = Store::ACTION_PERFORM_SAVE;
        
        foreach ($values as $key => $value) {
            $_POST[$key] = $value;
            $_REQUEST[$key] = $value;
        }
        
        $_GET[$storeName][Store::ACTION_KEY_IN_REQUEST] = $action;
        $_REQUEST[$storeName][Store::ACTION_KEY_IN_REQUEST] = $action;
        
        if ($primaryKeyValue) {
            $key = Store::PRIMARY_KEY_IN_REQUEST;
            $_GET[$storeName][$key] = $primaryKeyValue;
            $_REQUEST[$storeName][$key] = $primaryKeyValue;
        }
    } // end preparePostByStoreName

    public static function prepareChildRelationPostByStoreName(
        $storeName,
        $idParent,
        $values,
        $parentValues = array(),
        $primaryKeyValue = false
    )
    {
        // FIXME: Refactoring session key festi
        $_SESSION['festi']['DB_'.$storeName.'_caption'] = 'Parent Store '.rand(1, 2000);
        $_SESSION['festi']['DB_'.$storeName.'_PARENT'] = $idParent;

        static::preparePostByStoreName($storeName, $values, $primaryKeyValue);

        $insertToken = md5("test_".rand(1, 200000));
        $_REQUEST[$storeName][Store::REQUEST_KEY_INSERT_TOKEN] = $insertToken;

        $_SESSION['festi']['insert'][$insertToken] = $parentValues;
    }
    
    public static function preparePrimaryKey(&$store, $primaryKeyValue)
    {
        $storeName = $store->getIdent();
        
        $_GET[$storeName][Store::PRIMARY_KEY_IN_REQUEST] = $primaryKeyValue;
        $_REQUEST[$storeName][Store::PRIMARY_KEY_IN_REQUEST] = $primaryKeyValue;
        
        unset($_POST[$storeName][Store::ACTION_PERFORM_KEY_IN_POST]);
        
        $store->setRequest($_REQUEST[$storeName]);
    } // end preparePrimaryKey
    
    protected function doDump(string $name, ?string $path = null): void
    {
        $db = $this->getDbConnection();

        $type = $db->getDatabaseType();

        if (!$path) {
            $path = FS_RESOURCES_ROOT.'dump'.DIRECTORY_SEPARATOR;
        }
    
        $path .= $name.".".$type.".sql";

        $content = file_get_contents($path);

        // XXX: Fix \n in empty string
        $rows = array_filter(
            explode(";", $content),
            function($var) {
                return (bool) trim($var);
            }
        );

        foreach ($rows as $sql) {
            $db->query($sql);
        }
    } // end doDump

    /**
     * Returns a MockObject for a plugin.
     *
     * @param $pluginName
     * @param array $options
     * @return mixed
     * @throws SystemException
     */
    public function getPluginMockBuilder(
        $pluginName, $methods = array(), $options = array()
    )
    {
        $core = Core::getInstance();

        $plugin = &$core->getPluginInstance($pluginName, $options);

        $pluginOptions = $plugin->getOptions();

        $pluginPath = $core->getOption("plugins_path");

        $classPostfix = 'Plugin';
        if (array_key_exists('classPostfix', $options)) {
            $classPostfix = $options['classPostfix'];
        }

        $className = $pluginName.$classPostfix;

        $pluginPath = $pluginPath.$pluginName.DIRECTORY_SEPARATOR.
                      $className.'.php';

        require_once $pluginPath;

        $pluginMock = $this->getMockBuilder($className)
            //->setConstructorArgs(array())
            ->setMethods($methods)
            ->getMock();

        $pluginMock->setOptions($pluginOptions);

        $plugin = $pluginMock;
        $plugin->onInit();

        return $pluginMock;
    } // end getPluginMockBuilder
    
    /**
     * Returns a MockObject for a plugin.
     *
     * @param $actionName
     * @param array $methods
     * @param array $options
     * @return mixed
     * @throws SystemException
     */
    public function getActionMockBuilder(
        $actionName, $methods = array(), $options = array()
    )
    {
        $core = Core::getInstance();

        $corePath = $core->getOption("core_path");

        $classPostfix = 'Action';
        if (array_key_exists('classPostfix', $options)) {
            $classPostfix = $options['classPostfix'];
        }

        $className = $actionName.$classPostfix;

        $actionPath = $corePath.'store'.DIRECTORY_SEPARATOR.'action'.
                      DIRECTORY_SEPARATOR.$className.'.php';

        require_once $actionPath;
        
        $store = $this->getDefaultStore();
        
        if (array_key_exists('storeInstance', $options)) {
            $store = $options['storeInstance'];
        }
        
        $actionMock = $this->getMockBuilder($className)
            ->setConstructorArgs(array(&$store))
            ->setMethods($methods)
            ->getMock();

        return $actionMock;
    } // end getActionMockBuilder
    
    public function doTestStore($storeName, $params = array(), $values = array())
    {
        $core = Core::getInstance();
        $store = $core->createStoreInstance($storeName, $params);
        
        $values = $this->getPreparedValues($store, $values);

        $actions = &$store->getModel()->getActions();
        
        $blackList = array(
            'parent',
            'child'
        );
        
        foreach ($actions as $actionName => $actionData) {
            if (in_array($actionName, $blackList)) {
                continue;
            }
            
            $methodName = "onTest".ucfirst($actionName)."StoreAction";
            if (!method_exists($this, $methodName)) {
                $this->fail(sprintf("Test Method '%s' Not Fond", $methodName));
            }
            $this->$methodName($store, $values);
        }
    } // end doTestStore
    
    protected function addDatabaseData(Store $store, $values)
    {
        $tableName = $store->getModel()->getName();
        $core = Core::getInstance();

        return $core->db->insert($tableName, $values);
    } // end addDatabaseData
    
    protected function onTestInsertStoreAction(Store $store, $values)
    {
        $values = $this->getPreparedValues($store, $values);
        
        $action = $store->createActionInstance(Store::ACTION_INSERT);
        $this->assertInstanceOf('InsertAction', $action);
        
        static::preparePostForStore(
            Store::ACTION_INSERT,
            $values,
            $store
        );
        
        $response = new Response();
        
        $action->onStart($response);
        
        $search = $this->getSearch($store, $values);

        $data = $store->loadRow($search);
    
        $fields = $store->getModel()->getFields();
    
        foreach ($fields as $field) {
            $key = $field->getName();

            if (!$key || !array_key_exists($key, $values)) {
                continue;
            }
        
            $value = $values[$key];
            $regExp = "#".$value."#Umis";
            $this->assertMatchesRegularExpression($regExp, $data[$key], "Data is Not Found");
        }
        
        return $response;
    } // end onTestInsertStoreAction
    
    protected function getSearch(Store $store, $values)
    {
        $search = array();
        $storeName = $store->getName();
        foreach ($values as $key => $value) {
            if (strpos($key, '.') === true) {
                $search[$key] = $value;
                continue;
            }
            $search[$storeName.".".$key] = $value;
        }
        
        return $search;
    } // end getSearch
    
    protected function onTestListStoreAction(Store $store, $values)
    {
        $id = $this->addDatabaseData($store, $values);
        static::cleanFiltersForStore($store);
        
        $session = &$store->getSession();
        $storeName = $store->getName();
        unset($session['DB_'.$storeName.'_PARENT_ROW']);
        unset($session['DB_'.$storeName.'_PARENT']);
        
        $action = $store->createActionInstance(Store::ACTION_LIST);
    
        $this->assertInstanceOf('ListAction', $action);
    
        $response = new Response();
    
        $action->onStart($response);

        $this->assertNotEmpty($response->content);
        $this->assertInstanceOf('ListAction', $response->storeAction);
        
        $fields = $store->getModel()->getFields();

        foreach ($fields as $field) {
            if ($field->get('hide') == 'true' || !$field->getName()) {
                continue;
            }
            
            $regExp = sprintf(
                "#e-db-table-row-item-%s-%s#Umis",
                $field->getName(),
                $id
            );
            
            $this->assertMatchesRegularExpression($regExp, $response->content);
        }
    } // end onTestListStoreAction
    
    protected function onTestEditStoreAction(Store $store, $values)
    {
        $id = $this->addDatabaseData($store, $values);
        
        $action = $store->createActionInstance(Store::ACTION_EDIT);

        $this->assertInstanceOf('EditAction', $action);

        $newValues = $this->getPreparedValues($store, $values);
        
        $this->assertNotEquals(
            md5(json_encode($values)),
            md5(json_encode($newValues))
        );
        
        FestiTestCase::preparePostForStore(
            Store::ACTION_EDIT,
            $newValues,
            $store,
            $id
        );
    
        $response = new Response();

        $action->onStart($response);

        $search = $this->getSearch($store, $newValues);
        
        $data = $store->loadRow($search);
        
        $this->assertNotEmpty($data);
        $this->assertEquals($data['id'], $id);
    } // end onTestEditStoreAction
    
    protected function onTestRemoveStoreAction(Store $store, $values)
    {
        $id = $this->addDatabaseData($store, $values);
        
        $search = array(
            $store->getName().'.id' => $id
        );

        $data = $store->loadRow($search);

        $this->assertEquals($id, $data['id']);
        
        $action = $store->createActionInstance(Store::ACTION_REMOVE);
        $this->assertInstanceOf('RemoveAction', $action);
        
        FestiTestCase::preparePostForStore(
            Store::ACTION_REMOVE,
            array(),
            $store,
            $id
        );

        $store->addAllowedTableRow($id);
        
        $response = new Response();
    
        $action->onStart($response);

        $data = $store->loadRow($search);
        
        $this->assertEmpty($data);
    } // end onTestRemoveStoreAction
    
    protected function onTestInfoStoreAction(Store $store, $values)
    {
        $id = $this->addDatabaseData($store, $values);
    
        $action = $store->createActionInstance(Store::ACTION_INFO);
    
        $this->assertInstanceOf('InfoAction', $action);
        
        FestiTestCase::preparePostForStore(
            Store::ACTION_INFO,
            array(),
            $store,
            $id
        );
        
        $response = new Response();

        $action->onStart($response);

        ob_start();
    
        $response->send();
        $content = ob_get_contents();
    
        ob_end_clean();

        foreach ($values as $key => $value) {
            $value = $this->getPreparedValueByType($store, $key, $value);
            $this->assertMatchesRegularExpression(
                "#".$value."#Umis",
                $content,
                "Not found Info form."
            );
        }
    } // end onTestInfoStoreAction
    
    protected function getPreparedValueByType($store, $name, $value)
    {
        $fields = $store->getModel()->getFields();
        foreach ($fields as $field) {
            if ($field->getType() == 'datetime' && $field->getName() == $name) {
                $format = $field->get('format');
                $value = strftime($format, strtotime($value));
            }
        }
        return $value;
    } // end getPreparedValueByType
    
    protected function onTestBatchInsertStoreAction(Store $store, $values)
    {
        $action = $store->createActionInstance(Store::ACTION_BATCH_INSERT);
        
        $this->assertInstanceOf('BatchInsertAction', $action);
        
        $countData = count($store->load());
        
        $batchValues = array(
            $this->getPreparedValues($store, $values),
            $this->getPreparedValues($store, $values)
        );

        $response = new Response();
        
        $_POST = array();
        $_REQUEST = $batchValues;

        $action->onUpdate($response);
        
        $this->assertCount(
            $countData+2,
            $store->load(),
            "Must be more than two entries"
        );
        
        $data = $store->loadRow($batchValues[0]);
        $this->assertNotEmpty($data);
        $data = $store->loadRow($batchValues[1]);
        $this->assertNotEmpty($data);
    } // end onTestBatchInsertStoreAction
    
    protected function onTestPluginStoreAction($store, $values)
    {
        $this->fail('Override Method "onTestPluginStoreAction"');
    } // end onTestPluginStoreAction
    
    protected function onTestCsvImportStoreAction($store, $values)
    {
        $this->fail('Override Method "onTestCsvImportStoreAction"');
    } // end onTestCsvImportStoreAction
    
    protected function getPreparedValues(Store $store, $values)
    {
        $fields = $store->getModel()->getFields();
        $storeName = $store->getName();
        $randomValues = array();
        
        foreach ($fields as $field) {
            $type = $field->getType();
            $key = $field->getName();
            if ($field->get('readonly') == 'true') {
                continue;
            }
            switch ($type) {
                case 'textarea':
                    $randomValues[$key] = $this->getRandomString(200);
                    break;
                case 'datetime':
                    $randomValues[$key] = date("Y-m-d H:i:s");
                    break;
                case 'text':
                case 'composite':
                    $randomValues[$key] = $this->getRandomString();
                    break;
                case 'number':
                    $randomValues[$key] = $this->getRandomNumber();
                    break;
            }
        }
        
        return array_merge($values, $randomValues);
    } // end getPreparedValues
    
    protected function getRandomString($length = 20)
    {
        $characters = 'abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ';
        $randomString = "";
        $charactersLength = strlen($characters);
        
        for ($i = 0; $i < $length; $i++) {
            $randomString .= $characters[rand(0, $charactersLength - 1)];
        }
        
        return $randomString;
    } // end getRandomString
    
    protected function getRandomNumber($min = 1, $max = 2000000): int
    {
        return mt_rand($min, $max);
    } // end getRandomNumber

    public static function prepareEnvironment(): Core
    {
        define('FS_TESTS_ROOT', __DIR__.DIRECTORY_SEPARATOR);
        define('FS_RESOURCES_ROOT', FS_TESTS_ROOT.'Resources'.DIRECTORY_SEPARATOR);
        define('FS_PROJECT_ROOT', realpath(__DIR__.'/../').DIRECTORY_SEPARATOR);
        define(
            'FS_THEME_ROOT',
            FS_RESOURCES_ROOT.'themes'.DIRECTORY_SEPARATOR.'default'.
            DIRECTORY_SEPARATOR
        );

        define('STORE_DEFAULT', 'settings');

        if (intval(ini_get('short_open_tag')) > 0) {
            $message = __(
                'short_open_tag have to be disabled [%s]',
                php_ini_loaded_file()
            );
            throw new Exception($message);
        }

        require_once FS_TESTS_ROOT.'FestiTestCase.php';
        require_once FS_TESTS_ROOT.'ThemeTestCase.php';
        require_once FS_TESTS_ROOT.'TestUtils.php';

        $options = array(
            Core::OPTION_PATH                        => FS_PROJECT_ROOT,
            Core::OPTION_ENGINE_FOLDER               => '',
            Core::OPTION_DEFS_PATH                   => FS_RESOURCES_ROOT.'defs'.DIRECTORY_SEPARATOR,
            Core::OPTION_THEME_PATH                  => FS_THEME_ROOT,
            Core::OPTION_THEME_TEMPLATE_PATH         => FS_THEME_ROOT.'templates'.DIRECTORY_SEPARATOR,
            Core::OPTION_PLUGINS_PATH                => FS_RESOURCES_ROOT.'plugins'.DIRECTORY_SEPARATOR,
            Core::OPTION_GLOBAL_STORE_EXCEPTION_MODE => true,
        );

        $core = Core::getInstance($options);

        $dbType = getenv('DB_TYPE');
        if (!$dbType) {
            $dbType = DataAccessObject::TYPE_MYSQL;
        }

        $config = array(
            'DB_USER' => 'developer',
            'DB_PASSWORD' => 'developertest',
            'DB_NAME' => 'festi_tests',
            'DB_HOST' => 'localhost',
            'DB_PORT' => null
        );

        foreach ($config as $variableName => &$value) {
            $envValue = getenv($variableName);
            if ($envValue) {
                $value = $envValue;
            }
        }

        $dbConfigs = array(
            DataAccessObject::TYPE_MYSQL => array(
                'DB_DSN' => 'mysql:dbname='.$config['DB_NAME'].';host='.$config['DB_HOST'].
                            ($config['DB_PORT'] ? ':'.$config['DB_PORT'] : ''),
                'DB_USER' => $config['DB_USER'],
                'DB_PASSWD' => $config['DB_PASSWORD'],
                'DB_NAME' => $config['DB_NAME']
            ),
            DataAccessObject::TYPE_PGSQL => array(
                'DB_DSN' => 'pgsql:dbname='.$config['DB_NAME'].';host='.$config['DB_HOST'].
                            ($config['DB_PORT'] ? ':'.$config['DB_PORT'] : ''),
                'DB_USER' => $config['DB_USER'],
                'DB_PASSWD' => $config['DB_PASSWORD'],
                'DB_NAME' => $config['DB_NAME']
            ),
            DataAccessObject::TYPE_MSSQL => array(
                'DB_DSN' => 'sqlsrv:Server='.$config['DB_HOST'].';Database='.$config['DB_NAME'],
                'DB_USER' => $config['DB_USER'],
                'DB_PASSWD' => 'developertest1@',
                'DB_NAME' => $config['DB_NAME']
            )
        );

        if (!array_key_exists($dbType, $dbConfigs)) {
            throw new Exception("Undefined database type");
        }

        define('TEST_DB_TYPE', $dbType);

        $dbConfig = $dbConfigs[$dbType];
        $GLOBALS['config']['db'] = $dbConfig;

        $core->db = TestUtils::getDbConnection($dbConfig);

        TestUtils::cleanDatabase();

        $params = array(
            'auth'       => 'yes',
            'auth_id'    => '1',
            'auth_login' => 'admin',
            'auth_role'  => '1'
        );
        $core->user = new DefaultUser($params);

        $systemPlugin = $core->getPluginInstance('System');

        $core->setSystemPlugin($systemPlugin);

        return $core;
    }
}
