<?php

interface IObjectDriver
{
    public function quoteTableName(string $name): string;
    public function quoteColumnName(string $name): string;

    public function getErrorCode($code): int;

    public function createSelectQuery(
        array $columns,
        string $from,
        ?array $joins = null,
        ?array $where = null,
        ?array $orderBy = null,
        ?int $limit = null,
        ?int $offset = null,
        ?array $groupBy = null,
        ?array $having = null
    ): string;

    /**
     * Returns the generated SQL query to insert data
     *
     * @param IDataAccessObject $object
     * @param string $table
     * @param array $values
     * @param bool $isUpdateDuplicate
     * @return string SQL string
     */
    public function createInsertQuery(
        IDataAccessObject $object,
        string $table,
        array $values,
        bool $isUpdateDuplicate = false
    ): string;

    public function getSplitOnPages(IDataAccessObject $object, string $query, int $col, int $page): array;
    public function getTableIndexes(IDataAccessObject $object, string $tableName): array;

    /**
     * Remove all tables and constraints from database.
     *
     * @param IDataAccessObject $object
     * @return bool
     */
    public function cleanDatabase(IDataAccessObject $object): bool;
}