<?php

class PgsqlObjectDriver extends AbstractObjectDriver
{
    public function __construct($db)
    {
        $db->query("SET NAMES 'utf8'");
    } // end __construct

    public function quoteTableName(string $name): string
    {
        return '"'.$name.'"';
    } // end quoteTableName
    
    public function quoteColumnName(string $key): string
    {
        if ($this->isExpression($key)) {
            return $this->quoteExpressionColumnName($key);
        }
    
        $key = "\"".$key."\"";
        if (strpos($key, '.') !== false) {
            $key = str_replace(".", "\".\"", $key);
        }
        
        return $key;
    } // end quoteColumnName

    public function getTableIndexes(IDataAccessObject $object, string $tableName): array
    {
        $sql = "SELECT indexname as table, indexdef FROM pg_indexes WHERE tablename = ".$object->quote($tableName);

        return $object->getAll($sql);
    }

    public function cleanDatabase(IDataAccessObject $object): bool
    {
        parent::cleanDatabase($object);

        $sql = "SELECT DISTINCT pg_type.typname ".
            "FROM pg_type JOIN pg_enum ON pg_enum.enumtypid = pg_type.oid";

        $types = $object->getCol($sql);

        foreach ($types as $type) {
            $object->query("DROP TYPE ".$type);
        }

        $types = $object->getCol(" SELECT c.relname FROM pg_class c WHERE c.relkind = 'S'");

        foreach ($types as $type) {
            $object->query("DROP SEQUENCE ".$type);
        }

        return true;
    }

}