<?php

namespace core\orm;

use core\orm\relation\IRelation;

class Column
{
    private $_name;

    /**
     * @var bool
     */
    private bool $_isPrimaryKey = false;

    /**
     * @var bool
     */
    private bool $_isNullable = false;

    /**
     * @var ?IRelation
     */
    private ?IRelation $_relation = null;

    /**
     * @var string
     */
    private string $_getter;

    /**
     * @var string
     */
    private string $_setter;

    public function __construct(string $name)
    {
        $this->_name = $name;
    }

    public function getName(): string
    {
        return $this->_name;
    } // end getName

    public function setPrimaryKey(bool $isPrimary): void
    {
        $this->_isPrimaryKey = $isPrimary;
    }

    public function setNullable(bool $isNullable): void
    {
        $this->_isNullable = $isNullable;
    }

    public function isNullable(): bool
    {
        return $this->_isNullable;
    }

    public function isPrimaryKey(): bool
    {
        return $this->_isPrimaryKey;
    }

    public function setRelation(IRelation $relation): void
    {
        $this->_relation = $relation;
    } // end setRelation

    public function hasRelation(): bool
    {
        return !empty($this->_relation);
    } // end hasRelation

    public function getRelation(): IRelation
    {
        return $this->_relation;
    }

    public function setSetter(string $callback): void
    {
        $this->_setter = $callback;
    }

    public function setGetter(string $callback): void
    {
        $this->_getter = $callback;
    }

    /**
     * Returns the getter method name.
     *
     * @return string
     */
    public function getGetter(): string
    {
        return $this->_getter;
    } // end getGetter

    public function getSetter(): string
    {
        return $this->_setter;
    }
}