<?php

namespace core\orm;

class Table
{
    private $_name;

    /**
     * @var Column[]
     */
    private $_columns = array();

    /**
     * @var Column[]
     */
    private $_columnsWithRelation = array();

    /**
     * @var Column[]
     */
    private $_primaryKey = array();

    public function addColumn(Column $column): void
    {
        $key = $column->getName();
        if (array_key_exists($key, $this->_columns)) {
            throw new EntityException("Column ".$key." already exists into ".$this->getName());
        }

        if ($column->hasRelation()) {
            $this->_columnsWithRelation[$key] = $column;
        } else if ($column->isPrimaryKey()) {
            $this->_primaryKey[$key] = $column;
        } else {
            $this->_columns[$key] = $column;
        }
    } // end addColumn

    public function getName(): string
    {
        return $this->_name;
    }

    public function setName(string $name): void
    {
        $this->_name = $name;
    }

    /**
     * @return Column[]
     */
    public function getPrimaryKey(): array
    {
        return $this->_primaryKey;
    } // end getPrimaryKey

    /**
     * @return Column[]
     */
    public function getColumns(): array
    {
        return $this->_columns;
    } // end getColumns

    /**
     * @return Column[]
     */
    public function getColumnsWithRelation(): array
    {
        return $this->_columnsWithRelation;
    } // end getColumnsWithRelation




}