<?php

namespace core\orm\relation;

use core\orm\EntityException;
use core\orm\Table;

abstract class Relation implements IRelation
{
    private array $_options;

    /**
     * @var Table
     */
    private $_table;

    public function __construct(array $options)
    {
        $this->_options = $options;
    }

    public function getEntityClass(): string
    {
        return $this->getOption('entity');
    } // end getEntityClass

    public function setTable(Table $table): void
    {
        $this->_table = $table;
    } // end setTable

    public function getTable(): Table
    {
        return $this->_table;
    }

    public function getForeignKeyColumnName(): string
    {
        return $this->getOption('foreignKey');
    }

    public function getSourceKeyColumnName(): string
    {
        return $this->getOption('id');
    }

    protected function getOption($key): string
    {
        if (!array_key_exists($key, $this->_options)) {
            throw new EntityException("Undefined option ".$key." into ".get_class($this));
        }

        return $this->_options[$key];
    } // end getOption

}