<?php

interface ITheme
{
    public const EVENT_THEME_FETCH_GOOGLE_ANALYTICS = 'ThemeFetchGoogleAnalytics';
    public const EVENT_THEME_FETCH_MENU             = 'ThemeFetchMenu';
    public const EVENT_THEME_FETCH_HEADER_ELEMENT   = 'ThemeFetchHeaderElement';
    public const EVENT_THEME_FETCH_FOOTER           = 'ThemeFetchFooter';
    public const EVENT_THEME_HEAD_BOTTOM            = 'ThemeHeadBottom';
    public const EVENT_THEME_HEADER_USER_ACTIVITY   = 'ThemeHeaderUserActivity';
    public const EVENT_THEME_PROJECTS_LIST          = 'ThemeHeaderProjectsList';
    public const EVENT_THEME_TITLE                  = 'theme_title';
    public const EVENT_THEME_ON_PREPARE_LOGO        = 'event_theme_on_prepare_logo';

    /**
     * Fire Theme Event.
     *
     * @param string $themeEventName
     * @param array $target
     * @return array
     */
    public function doThemeEvent(string $themeEventName, array &$target = array()): array;

    /**
     * Returns content in main theme context.
     *
     * @param string $content
     * @param string $template
     * @return string
     */
    public function fetchMain(string $content, $template = 'main.phtml'): string;

    /**
     * Returns theme logo
     *
     * Dispatching ITheme::EVENT_THEME_ON_PREPARE_LOGO
     * Event target contains link and logo keys.
     * link - the link to which the logo leads
     * logo - logo src
     *
     * Also if event target contains content key,
     * function returns it`s target value.
     *
     * @return string
     */
    public function fetchLogo(): string;

    public function fetchMenu($area = false, $template = false): string;

    public function fetchHead(): string;

    public function fetchNotifications(): string;

    public function fetchThemeTitle(): string;

    public function fetchHeadBottom(): string;

    public function fetchGoogleAnalytics(): ?string;

    public function fetchUserActivities(): string;

    public function fetchProjectsList(): string;

    public function fetchFooter(): string;

    /**
     * Override theme options.
     *
     * @param array $options
     */
    public function setOptions(array $options): void;
}
