<?php

namespace ui;

use core\ITemplate;

class UIEngine implements IUIEngine
{
    private ITemplate $_template;

    public function __construct(ITemplate $template)
    {
        $this->_template = $template;
    }

    protected function createElement(string $className, string $templateName)
    {
        $instance = new $className();
        
        $instance->view = clone $this->_template;

        $instance->setTemplateName($templateName.".php");

        return $instance;
    }

    public function button(): IButton
    {
        return $this->createElement(\ButtonHui::class, 'button');
    }

    public function buttons(): IButtons
    {
        return $this->createElement(\ButtonsHui::class, 'buttons');
    }

    public function dialog(): IDialog
    {
        return $this->createElement(\DialogHui::class, 'dialog');
    }

    public function input(): IInput
    {
        return $this->createElement(\InputHui::class, 'input');
    }

    public function select(): ISelect
    {
        return $this->createElement(\SelectHui::class, 'select');
    }

    public function widget(): IWidget
    {
        return $this->createElement(\WidgetHui::class, 'widget');
    }

    public function block(): IBlock
    {
        return $this->createElement(\BlockHui::class, 'block');
    }

    public function breadcrumbs(): IBreadcrumbs
    {
        return $this->createElement(\BreadcrumbsHui::class, 'breadcrumbs');
    }

    public function sparks(): ISparks
    {
        return $this->createElement(\SparksHui::class, 'sparks');
    }

    public function message(): IMessage
    {
        return $this->createElement(\MessageHui::class, 'message');
    }

    public function datalist(): IDataList
    {
        return $this->createElement(\DataListHui::class, 'dl');
    }
}