<?php

use ui\IInput;
use ui\ISelect;

class UITest extends PHPUnit\Framework\TestCase
{
    protected Core $core;

    protected function setUp(): void
    {
        $options = array(
            Core::OPTION_THEME_PATH          => FS_THEME_ROOT,
            Core::OPTION_THEME_TEMPLATE_PATH => FS_THEME_ROOT.'templates'.DIRECTORY_SEPARATOR,
        );

        $this->core = Core::getInstance($options);
    } // end setUp

    public function testUiComponents()
    {
        $plugin = $this->core->getPluginInstance(TestPlugin::class);
        assert($plugin instanceof TestPlugin);

        $button = $plugin->ui->button()->setCaption('test')
                                        ->setId('test-id')
                                        ->setUrl('/')
                                        ->setCSS('css-name');

        $this->assertNotEmpty($button->fetch());

        //
        $messageInput = $plugin->ui->input()->setName('message')->setType('textarea');
        $this->assertInstanceOf(IInput::class, $messageInput);

        $content = $plugin->ui->dialog()->setTitle(__('Close the User Review #%s'))
            ->addField(__('Message'), $messageInput)
            ->setUrl('/')
            ->fetch();

        $this->assertNotEmpty($content);

        //
        $content = $plugin->ui->block()->setTitle(__('Summary'))->setContent('Content')->fetch();
        $this->assertNotEmpty($content);

        //
        $content = $plugin->ui->widget()
            ->setTitle(__('Invoice Creation'))
            ->setContent("Content")
            ->fetch();

        $this->assertNotEmpty($content);

        //
        $items = array(
            __('Dashboard') => '/',
            __('Report') => false
        );

        $content = $plugin->ui->breadcrumbs()
            ->setItems($items)
            ->setRight("Content")
            ->fetch();

        $this->assertNotEmpty($content);

        //
        $sparks = array(
            array(
                'caption' => __('Month'),
                'total' => '03-2019'
            )
        );

        $sparksContent = $plugin->ui->sparks()->setItems($sparks)->fetch();
        $this->assertNotEmpty($sparksContent);

        //
        $message = $plugin->ui->message()->setType('info')->setMessage(__('Profit'))->fetch();
        $this->assertNotEmpty($message);

        //
        $columns = array(
            'name' => __('Name')
        );

        $data = array(
            'name' => 'Jack'
        );

        $content = $plugin->ui->datalist()->setColumns($columns)->setRows($data)->fetch();
        $this->assertNotEmpty($content);
    }
    
    public function testTwoDifferentComponents()
    {
        $plugin = $this->core->getPluginInstance(TestPlugin::class);
        assert($plugin instanceof TestPlugin);
        
        $messageInput = $plugin->ui->input()->setName('message');
        $this->assertInstanceOf(IInput::class, $messageInput);
        
        $select = $plugin->ui->select()->setName('select')->setType('select')->addOption('key', 'value');
        $this->assertInstanceOf(ISelect::class, $select);
    
        $content = $plugin->ui->dialog()->setTitle(__('Some Title'))
            ->addField(__('Message'), $messageInput)
            ->addField(__('Select'), $select)
            ->setUrl('/')
            ->fetch();
    
        $this->assertNotEmpty($content);
        
        $this->assertMatchesRegularExpression("#<option value=\"key\" #Umis", $content);
        $this->assertMatchesRegularExpression("#<input type=\"text\" #Umis", $content);
    }
}

class TestPlugin extends DisplayPlugin
{
}