<?php

class FestiCoreStandalone
{
    const EVENT_ON_INIT = "core_standalone_init";

    private static ?IEngineCompatibility $_instance = null;

    /**
     * Return true if instance has been init.
     *
     * @return bool
     */
    public static function isInit(): bool
    {
        return !empty(static::$_instance);
    } // end isInit

    public static function init(array $options = array()): void
    {
        if (static::$_instance) {
            throw new FestiCoreStandaloneException(
                'FestiCoreStandalone already defined.'
            );
        }
        
        static::$_instance = static::_createEngineCompatibilityInstance($options);

        $event = new FestiEvent(static::EVENT_ON_INIT);
        Core::getInstance()->dispatchEvent($event);
    } // end init

    public static function install($options = array())
    {
        if (!static::$_instance) {
            throw new FestiCoreStandaloneException(
                'FestiCoreStandalone hasn\'t inited. FestiCoreStandalone::init()'
            );
        }
        
        static::$_instance->install($options);
    } // end install
    
    public static function bind($options = array())
    {
        if (!static::$_instance) {
            throw new FestiCoreStandaloneException(
                'FestiCoreStandalone isn\'t init. Use FestiCoreStandalone::init'
            );
        }
        
        static::$_instance->bind($options);
        
    } // end bind
    
    private static function _createEngineCompatibilityInstance(array $options): IEngineCompatibility
    {
        $className = null;

        if (!empty($GLOBALS['wp_version'])) {
            $className = WordpressEngineCompatibility::class;
        }

        if (!$className) {
            throw new FestiCoreStandaloneException("Undefined Engine");
        }

        return new $className($options);
    } // end _createEngineCompatibilityInstance
}